# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Simulation/Drive.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Simulation/Drive
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from ._frozen import FrozenClass

from ._check import InitUnKnowClassError


class Drive(FrozenClass):
    """Abstract Drive class"""

    VERSION = 1

    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self, Umax=800, Imax=800, is_current=False, init_dict=None, init_str=None
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "Umax" in list(init_dict.keys()):
                Umax = init_dict["Umax"]
            if "Imax" in list(init_dict.keys()):
                Imax = init_dict["Imax"]
            if "is_current" in list(init_dict.keys()):
                is_current = init_dict["is_current"]
        # Set the properties (value check and convertion are done in setter)
        self.parent = None
        self.Umax = Umax
        self.Imax = Imax
        self.is_current = is_current

        # The class is frozen, for now it's impossible to add new properties
        self._freeze()

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        Drive_str = ""
        if self.parent is None:
            Drive_str += "parent = None " + linesep
        else:
            Drive_str += "parent = " + str(type(self.parent)) + " object" + linesep
        Drive_str += "Umax = " + str(self.Umax) + linesep
        Drive_str += "Imax = " + str(self.Imax) + linesep
        Drive_str += "is_current = " + str(self.is_current) + linesep
        return Drive_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False
        if other.Umax != self.Umax:
            return False
        if other.Imax != self.Imax:
            return False
        if other.is_current != self.is_current:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()
        if other._Umax != self._Umax:
            diff_list.append(name + ".Umax")
        if other._Imax != self._Imax:
            diff_list.append(name + ".Imax")
        if other._is_current != self._is_current:
            diff_list.append(name + ".is_current")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object
        S += getsizeof(self.Umax)
        S += getsizeof(self.Imax)
        S += getsizeof(self.is_current)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        Drive_dict = dict()
        Drive_dict["Umax"] = self.Umax
        Drive_dict["Imax"] = self.Imax
        Drive_dict["is_current"] = self.is_current
        # The class name is added to the dict for deserialisation purpose
        Drive_dict["__class__"] = "Drive"
        return Drive_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.Umax = None
        self.Imax = None
        self.is_current = None

    def _get_Umax(self):
        """getter of Umax"""
        return self._Umax

    def _set_Umax(self, value):
        """setter of Umax"""
        check_var("Umax", value, "float", Vmin=0)
        self._Umax = value

    Umax = property(
        fget=_get_Umax,
        fset=_set_Umax,
        doc=u"""Maximum RMS voltage of the Drive

        :Type: float
        :min: 0
        """,
    )

    def _get_Imax(self):
        """getter of Imax"""
        return self._Imax

    def _set_Imax(self, value):
        """setter of Imax"""
        check_var("Imax", value, "float", Vmin=0)
        self._Imax = value

    Imax = property(
        fget=_get_Imax,
        fset=_set_Imax,
        doc=u"""Maximum RMS current of the Drive

        :Type: float
        :min: 0
        """,
    )

    def _get_is_current(self):
        """getter of is_current"""
        return self._is_current

    def _set_is_current(self, value):
        """setter of is_current"""
        check_var("is_current", value, "bool")
        self._is_current = value

    is_current = property(
        fget=_get_is_current,
        fset=_set_is_current,
        doc=u"""True to generate current waveform, False for voltage

        :Type: bool
        """,
    )



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Simulation/MagFEMM.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Simulation/MagFEMM
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .Magnetics import Magnetics

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Simulation.MagFEMM.comp_flux_airgap import comp_flux_airgap
except ImportError as error:
    comp_flux_airgap = error

try:
    from ..Methods.Simulation.MagFEMM.get_path_save import get_path_save
except ImportError as error:
    get_path_save = error

try:
    from ..Methods.Simulation.MagFEMM.solve_FEMM import solve_FEMM
except ImportError as error:
    solve_FEMM = error

try:
    from ..Methods.Simulation.MagFEMM.get_meshsolution import get_meshsolution
except ImportError as error:
    get_meshsolution = error

try:
    from ..Methods.Simulation.MagFEMM.get_path_save_fem import get_path_save_fem
except ImportError as error:
    get_path_save_fem = error

try:
    from ..Methods.Simulation.MagFEMM.solve_FEMM_parallel import solve_FEMM_parallel
except ImportError as error:
    solve_FEMM_parallel = error


from ._check import InitUnKnowClassError
from .DXFImport import DXFImport


class MagFEMM(Magnetics):
    """Magnetic module: Finite Element model with FEMM"""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Simulation.MagFEMM.comp_flux_airgap
    if isinstance(comp_flux_airgap, ImportError):
        comp_flux_airgap = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use MagFEMM method comp_flux_airgap: "
                    + str(comp_flux_airgap)
                )
            )
        )
    else:
        comp_flux_airgap = comp_flux_airgap
    # cf Methods.Simulation.MagFEMM.get_path_save
    if isinstance(get_path_save, ImportError):
        get_path_save = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use MagFEMM method get_path_save: " + str(get_path_save)
                )
            )
        )
    else:
        get_path_save = get_path_save
    # cf Methods.Simulation.MagFEMM.solve_FEMM
    if isinstance(solve_FEMM, ImportError):
        solve_FEMM = property(
            fget=lambda x: raise_(
                ImportError("Can't use MagFEMM method solve_FEMM: " + str(solve_FEMM))
            )
        )
    else:
        solve_FEMM = solve_FEMM
    # cf Methods.Simulation.MagFEMM.get_meshsolution
    if isinstance(get_meshsolution, ImportError):
        get_meshsolution = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use MagFEMM method get_meshsolution: "
                    + str(get_meshsolution)
                )
            )
        )
    else:
        get_meshsolution = get_meshsolution
    # cf Methods.Simulation.MagFEMM.get_path_save_fem
    if isinstance(get_path_save_fem, ImportError):
        get_path_save_fem = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use MagFEMM method get_path_save_fem: "
                    + str(get_path_save_fem)
                )
            )
        )
    else:
        get_path_save_fem = get_path_save_fem
    # cf Methods.Simulation.MagFEMM.solve_FEMM_parallel
    if isinstance(solve_FEMM_parallel, ImportError):
        solve_FEMM_parallel = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use MagFEMM method solve_FEMM_parallel: "
                    + str(solve_FEMM_parallel)
                )
            )
        )
    else:
        solve_FEMM_parallel = solve_FEMM_parallel
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        Kmesh_fineness=1,
        Kgeo_fineness=1,
        type_calc_leakage=0,
        file_name="",
        FEMM_dict_enforced=-1,
        is_get_meshsolution=False,
        is_save_meshsolution_as_file=False,
        is_sliding_band=True,
        transform_list=-1,
        rotor_dxf=None,
        stator_dxf=None,
        import_file=None,
        is_close_femm=True,
        nb_worker=1,
        Rag_enforced=None,
        is_remove_slotS=False,
        is_remove_slotR=False,
        is_remove_vent=False,
        is_mmfs=True,
        is_mmfr=True,
        type_BH_stator=0,
        type_BH_rotor=0,
        is_periodicity_t=False,
        is_periodicity_a=False,
        angle_stator_shift=0,
        angle_rotor_shift=0,
        logger_name="Pyleecan.Magnetics",
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "Kmesh_fineness" in list(init_dict.keys()):
                Kmesh_fineness = init_dict["Kmesh_fineness"]
            if "Kgeo_fineness" in list(init_dict.keys()):
                Kgeo_fineness = init_dict["Kgeo_fineness"]
            if "type_calc_leakage" in list(init_dict.keys()):
                type_calc_leakage = init_dict["type_calc_leakage"]
            if "file_name" in list(init_dict.keys()):
                file_name = init_dict["file_name"]
            if "FEMM_dict_enforced" in list(init_dict.keys()):
                FEMM_dict_enforced = init_dict["FEMM_dict_enforced"]
            if "is_get_meshsolution" in list(init_dict.keys()):
                is_get_meshsolution = init_dict["is_get_meshsolution"]
            if "is_save_meshsolution_as_file" in list(init_dict.keys()):
                is_save_meshsolution_as_file = init_dict["is_save_meshsolution_as_file"]
            if "is_sliding_band" in list(init_dict.keys()):
                is_sliding_band = init_dict["is_sliding_band"]
            if "transform_list" in list(init_dict.keys()):
                transform_list = init_dict["transform_list"]
            if "rotor_dxf" in list(init_dict.keys()):
                rotor_dxf = init_dict["rotor_dxf"]
            if "stator_dxf" in list(init_dict.keys()):
                stator_dxf = init_dict["stator_dxf"]
            if "import_file" in list(init_dict.keys()):
                import_file = init_dict["import_file"]
            if "is_close_femm" in list(init_dict.keys()):
                is_close_femm = init_dict["is_close_femm"]
            if "nb_worker" in list(init_dict.keys()):
                nb_worker = init_dict["nb_worker"]
            if "Rag_enforced" in list(init_dict.keys()):
                Rag_enforced = init_dict["Rag_enforced"]
            if "is_remove_slotS" in list(init_dict.keys()):
                is_remove_slotS = init_dict["is_remove_slotS"]
            if "is_remove_slotR" in list(init_dict.keys()):
                is_remove_slotR = init_dict["is_remove_slotR"]
            if "is_remove_vent" in list(init_dict.keys()):
                is_remove_vent = init_dict["is_remove_vent"]
            if "is_mmfs" in list(init_dict.keys()):
                is_mmfs = init_dict["is_mmfs"]
            if "is_mmfr" in list(init_dict.keys()):
                is_mmfr = init_dict["is_mmfr"]
            if "type_BH_stator" in list(init_dict.keys()):
                type_BH_stator = init_dict["type_BH_stator"]
            if "type_BH_rotor" in list(init_dict.keys()):
                type_BH_rotor = init_dict["type_BH_rotor"]
            if "is_periodicity_t" in list(init_dict.keys()):
                is_periodicity_t = init_dict["is_periodicity_t"]
            if "is_periodicity_a" in list(init_dict.keys()):
                is_periodicity_a = init_dict["is_periodicity_a"]
            if "angle_stator_shift" in list(init_dict.keys()):
                angle_stator_shift = init_dict["angle_stator_shift"]
            if "angle_rotor_shift" in list(init_dict.keys()):
                angle_rotor_shift = init_dict["angle_rotor_shift"]
            if "logger_name" in list(init_dict.keys()):
                logger_name = init_dict["logger_name"]
        # Set the properties (value check and convertion are done in setter)
        self.Kmesh_fineness = Kmesh_fineness
        self.Kgeo_fineness = Kgeo_fineness
        self.type_calc_leakage = type_calc_leakage
        self.file_name = file_name
        self.FEMM_dict_enforced = FEMM_dict_enforced
        self.is_get_meshsolution = is_get_meshsolution
        self.is_save_meshsolution_as_file = is_save_meshsolution_as_file
        self.is_sliding_band = is_sliding_band
        self.transform_list = transform_list
        self.rotor_dxf = rotor_dxf
        self.stator_dxf = stator_dxf
        self.import_file = import_file
        self.is_close_femm = is_close_femm
        self.nb_worker = nb_worker
        self.Rag_enforced = Rag_enforced
        # Call Magnetics init
        super(MagFEMM, self).__init__(
            is_remove_slotS=is_remove_slotS,
            is_remove_slotR=is_remove_slotR,
            is_remove_vent=is_remove_vent,
            is_mmfs=is_mmfs,
            is_mmfr=is_mmfr,
            type_BH_stator=type_BH_stator,
            type_BH_rotor=type_BH_rotor,
            is_periodicity_t=is_periodicity_t,
            is_periodicity_a=is_periodicity_a,
            angle_stator_shift=angle_stator_shift,
            angle_rotor_shift=angle_rotor_shift,
            logger_name=logger_name,
        )
        # The class is frozen (in Magnetics init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        MagFEMM_str = ""
        # Get the properties inherited from Magnetics
        MagFEMM_str += super(MagFEMM, self).__str__()
        MagFEMM_str += "Kmesh_fineness = " + str(self.Kmesh_fineness) + linesep
        MagFEMM_str += "Kgeo_fineness = " + str(self.Kgeo_fineness) + linesep
        MagFEMM_str += "type_calc_leakage = " + str(self.type_calc_leakage) + linesep
        MagFEMM_str += 'file_name = "' + str(self.file_name) + '"' + linesep
        MagFEMM_str += "FEMM_dict_enforced = " + str(self.FEMM_dict_enforced) + linesep
        MagFEMM_str += (
            "is_get_meshsolution = " + str(self.is_get_meshsolution) + linesep
        )
        MagFEMM_str += (
            "is_save_meshsolution_as_file = "
            + str(self.is_save_meshsolution_as_file)
            + linesep
        )
        MagFEMM_str += "is_sliding_band = " + str(self.is_sliding_band) + linesep
        MagFEMM_str += (
            "transform_list = "
            + linesep
            + str(self.transform_list).replace(linesep, linesep + "\t")
            + linesep
        )
        if self.rotor_dxf is not None:
            tmp = self.rotor_dxf.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            MagFEMM_str += "rotor_dxf = " + tmp
        else:
            MagFEMM_str += "rotor_dxf = None" + linesep + linesep
        if self.stator_dxf is not None:
            tmp = (
                self.stator_dxf.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            )
            MagFEMM_str += "stator_dxf = " + tmp
        else:
            MagFEMM_str += "stator_dxf = None" + linesep + linesep
        MagFEMM_str += 'import_file = "' + str(self.import_file) + '"' + linesep
        MagFEMM_str += "is_close_femm = " + str(self.is_close_femm) + linesep
        MagFEMM_str += "nb_worker = " + str(self.nb_worker) + linesep
        MagFEMM_str += "Rag_enforced = " + str(self.Rag_enforced) + linesep
        return MagFEMM_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from Magnetics
        if not super(MagFEMM, self).__eq__(other):
            return False
        if other.Kmesh_fineness != self.Kmesh_fineness:
            return False
        if other.Kgeo_fineness != self.Kgeo_fineness:
            return False
        if other.type_calc_leakage != self.type_calc_leakage:
            return False
        if other.file_name != self.file_name:
            return False
        if other.FEMM_dict_enforced != self.FEMM_dict_enforced:
            return False
        if other.is_get_meshsolution != self.is_get_meshsolution:
            return False
        if other.is_save_meshsolution_as_file != self.is_save_meshsolution_as_file:
            return False
        if other.is_sliding_band != self.is_sliding_band:
            return False
        if other.transform_list != self.transform_list:
            return False
        if other.rotor_dxf != self.rotor_dxf:
            return False
        if other.stator_dxf != self.stator_dxf:
            return False
        if other.import_file != self.import_file:
            return False
        if other.is_close_femm != self.is_close_femm:
            return False
        if other.nb_worker != self.nb_worker:
            return False
        if other.Rag_enforced != self.Rag_enforced:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from Magnetics
        diff_list.extend(super(MagFEMM, self).compare(other, name=name))
        if other._Kmesh_fineness != self._Kmesh_fineness:
            diff_list.append(name + ".Kmesh_fineness")
        if other._Kgeo_fineness != self._Kgeo_fineness:
            diff_list.append(name + ".Kgeo_fineness")
        if other._type_calc_leakage != self._type_calc_leakage:
            diff_list.append(name + ".type_calc_leakage")
        if other._file_name != self._file_name:
            diff_list.append(name + ".file_name")
        if other._FEMM_dict_enforced != self._FEMM_dict_enforced:
            diff_list.append(name + ".FEMM_dict_enforced")
        if other._is_get_meshsolution != self._is_get_meshsolution:
            diff_list.append(name + ".is_get_meshsolution")
        if other._is_save_meshsolution_as_file != self._is_save_meshsolution_as_file:
            diff_list.append(name + ".is_save_meshsolution_as_file")
        if other._is_sliding_band != self._is_sliding_band:
            diff_list.append(name + ".is_sliding_band")
        if other._transform_list != self._transform_list:
            diff_list.append(name + ".transform_list")
        if (other.rotor_dxf is None and self.rotor_dxf is not None) or (
            other.rotor_dxf is not None and self.rotor_dxf is None
        ):
            diff_list.append(name + ".rotor_dxf None mismatch")
        elif self.rotor_dxf is not None:
            diff_list.extend(
                self.rotor_dxf.compare(other.rotor_dxf, name=name + ".rotor_dxf")
            )
        if (other.stator_dxf is None and self.stator_dxf is not None) or (
            other.stator_dxf is not None and self.stator_dxf is None
        ):
            diff_list.append(name + ".stator_dxf None mismatch")
        elif self.stator_dxf is not None:
            diff_list.extend(
                self.stator_dxf.compare(other.stator_dxf, name=name + ".stator_dxf")
            )
        if other._import_file != self._import_file:
            diff_list.append(name + ".import_file")
        if other._is_close_femm != self._is_close_femm:
            diff_list.append(name + ".is_close_femm")
        if other._nb_worker != self._nb_worker:
            diff_list.append(name + ".nb_worker")
        if other._Rag_enforced != self._Rag_enforced:
            diff_list.append(name + ".Rag_enforced")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from Magnetics
        S += super(MagFEMM, self).__sizeof__()
        S += getsizeof(self.Kmesh_fineness)
        S += getsizeof(self.Kgeo_fineness)
        S += getsizeof(self.type_calc_leakage)
        S += getsizeof(self.file_name)
        if self.FEMM_dict_enforced is not None:
            for key, value in self.FEMM_dict_enforced.items():
                S += getsizeof(value) + getsizeof(key)
        S += getsizeof(self.is_get_meshsolution)
        S += getsizeof(self.is_save_meshsolution_as_file)
        S += getsizeof(self.is_sliding_band)
        if self.transform_list is not None:
            for value in self.transform_list:
                S += getsizeof(value)
        S += getsizeof(self.rotor_dxf)
        S += getsizeof(self.stator_dxf)
        S += getsizeof(self.import_file)
        S += getsizeof(self.is_close_femm)
        S += getsizeof(self.nb_worker)
        S += getsizeof(self.Rag_enforced)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from Magnetics
        MagFEMM_dict = super(MagFEMM, self).as_dict(**kwargs)
        MagFEMM_dict["Kmesh_fineness"] = self.Kmesh_fineness
        MagFEMM_dict["Kgeo_fineness"] = self.Kgeo_fineness
        MagFEMM_dict["type_calc_leakage"] = self.type_calc_leakage
        MagFEMM_dict["file_name"] = self.file_name
        MagFEMM_dict["FEMM_dict_enforced"] = (
            self.FEMM_dict_enforced.copy()
            if self.FEMM_dict_enforced is not None
            else None
        )
        MagFEMM_dict["is_get_meshsolution"] = self.is_get_meshsolution
        MagFEMM_dict["is_save_meshsolution_as_file"] = self.is_save_meshsolution_as_file
        MagFEMM_dict["is_sliding_band"] = self.is_sliding_band
        MagFEMM_dict["transform_list"] = (
            self.transform_list.copy() if self.transform_list is not None else None
        )
        if self.rotor_dxf is None:
            MagFEMM_dict["rotor_dxf"] = None
        else:
            MagFEMM_dict["rotor_dxf"] = self.rotor_dxf.as_dict(**kwargs)
        if self.stator_dxf is None:
            MagFEMM_dict["stator_dxf"] = None
        else:
            MagFEMM_dict["stator_dxf"] = self.stator_dxf.as_dict(**kwargs)
        MagFEMM_dict["import_file"] = self.import_file
        MagFEMM_dict["is_close_femm"] = self.is_close_femm
        MagFEMM_dict["nb_worker"] = self.nb_worker
        MagFEMM_dict["Rag_enforced"] = self.Rag_enforced
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        MagFEMM_dict["__class__"] = "MagFEMM"
        return MagFEMM_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.Kmesh_fineness = None
        self.Kgeo_fineness = None
        self.type_calc_leakage = None
        self.file_name = None
        self.FEMM_dict_enforced = None
        self.is_get_meshsolution = None
        self.is_save_meshsolution_as_file = None
        self.is_sliding_band = None
        self.transform_list = None
        if self.rotor_dxf is not None:
            self.rotor_dxf._set_None()
        if self.stator_dxf is not None:
            self.stator_dxf._set_None()
        self.import_file = None
        self.is_close_femm = None
        self.nb_worker = None
        self.Rag_enforced = None
        # Set to None the properties inherited from Magnetics
        super(MagFEMM, self)._set_None()

    def _get_Kmesh_fineness(self):
        """getter of Kmesh_fineness"""
        return self._Kmesh_fineness

    def _set_Kmesh_fineness(self, value):
        """setter of Kmesh_fineness"""
        check_var("Kmesh_fineness", value, "float")
        self._Kmesh_fineness = value

    Kmesh_fineness = property(
        fget=_get_Kmesh_fineness,
        fset=_set_Kmesh_fineness,
        doc=u"""global coefficient to adjust mesh fineness in FEMM (1 : default , > 1 : finner , < 1 : less fine)

        :Type: float
        """,
    )

    def _get_Kgeo_fineness(self):
        """getter of Kgeo_fineness"""
        return self._Kgeo_fineness

    def _set_Kgeo_fineness(self, value):
        """setter of Kgeo_fineness"""
        check_var("Kgeo_fineness", value, "float")
        self._Kgeo_fineness = value

    Kgeo_fineness = property(
        fget=_get_Kgeo_fineness,
        fset=_set_Kgeo_fineness,
        doc=u"""global coefficient to adjust geometry fineness in FEMM (1 : default , > 1 : finner , < 1 : less fine)

        :Type: float
        """,
    )

    def _get_type_calc_leakage(self):
        """getter of type_calc_leakage"""
        return self._type_calc_leakage

    def _set_type_calc_leakage(self, value):
        """setter of type_calc_leakage"""
        check_var("type_calc_leakage", value, "int", Vmin=0, Vmax=1)
        self._type_calc_leakage = value

    type_calc_leakage = property(
        fget=_get_type_calc_leakage,
        fset=_set_type_calc_leakage,
        doc=u"""0 no leakage calculation /  1 calculation using single slot 

        :Type: int
        :min: 0
        :max: 1
        """,
    )

    def _get_file_name(self):
        """getter of file_name"""
        return self._file_name

    def _set_file_name(self, value):
        """setter of file_name"""
        check_var("file_name", value, "str")
        self._file_name = value

    file_name = property(
        fget=_get_file_name,
        fset=_set_file_name,
        doc=u"""Name of the file to save the FEMM model

        :Type: str
        """,
    )

    def _get_FEMM_dict_enforced(self):
        """getter of FEMM_dict_enforced"""
        return self._FEMM_dict_enforced

    def _set_FEMM_dict_enforced(self, value):
        """setter of FEMM_dict_enforced"""
        if type(value) is int and value == -1:
            value = dict()
        check_var("FEMM_dict_enforced", value, "dict")
        self._FEMM_dict_enforced = value

    FEMM_dict_enforced = property(
        fget=_get_FEMM_dict_enforced,
        fset=_set_FEMM_dict_enforced,
        doc=u"""To enforce user-defined values for FEMM main parameters 

        :Type: dict
        """,
    )

    def _get_is_get_meshsolution(self):
        """getter of is_get_meshsolution"""
        return self._is_get_meshsolution

    def _set_is_get_meshsolution(self, value):
        """setter of is_get_meshsolution"""
        check_var("is_get_meshsolution", value, "bool")
        self._is_get_meshsolution = value

    is_get_meshsolution = property(
        fget=_get_is_get_meshsolution,
        fset=_set_is_get_meshsolution,
        doc=u"""To save FEA and mesh for latter post-procesing 

        :Type: bool
        """,
    )

    def _get_is_save_meshsolution_as_file(self):
        """getter of is_save_meshsolution_as_file"""
        return self._is_save_meshsolution_as_file

    def _set_is_save_meshsolution_as_file(self, value):
        """setter of is_save_meshsolution_as_file"""
        check_var("is_save_meshsolution_as_file", value, "bool")
        self._is_save_meshsolution_as_file = value

    is_save_meshsolution_as_file = property(
        fget=_get_is_save_meshsolution_as_file,
        fset=_set_is_save_meshsolution_as_file,
        doc=u"""To save FEA and mesh as h5 files to save memory

        :Type: bool
        """,
    )

    def _get_is_sliding_band(self):
        """getter of is_sliding_band"""
        return self._is_sliding_band

    def _set_is_sliding_band(self, value):
        """setter of is_sliding_band"""
        check_var("is_sliding_band", value, "bool")
        self._is_sliding_band = value

    is_sliding_band = property(
        fget=_get_is_sliding_band,
        fset=_set_is_sliding_band,
        doc=u"""0 to desactivate the sliding band

        :Type: bool
        """,
    )

    def _get_transform_list(self):
        """getter of transform_list"""
        return self._transform_list

    def _set_transform_list(self, value):
        """setter of transform_list"""
        if type(value) is int and value == -1:
            value = list()
        check_var("transform_list", value, "list")
        self._transform_list = value

    transform_list = property(
        fget=_get_transform_list,
        fset=_set_transform_list,
        doc=u"""List of dictionnary to apply transformation on the machine surfaces. Key: label (to select the surface), type (rotate or translate), value (alpha or delta)

        :Type: list
        """,
    )

    def _get_rotor_dxf(self):
        """getter of rotor_dxf"""
        return self._rotor_dxf

    def _set_rotor_dxf(self, value):
        """setter of rotor_dxf"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "rotor_dxf"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = DXFImport()
        check_var("rotor_dxf", value, "DXFImport")
        self._rotor_dxf = value

        if self._rotor_dxf is not None:
            self._rotor_dxf.parent = self

    rotor_dxf = property(
        fget=_get_rotor_dxf,
        fset=_set_rotor_dxf,
        doc=u"""To use a dxf version of the rotor instead of build_geometry

        :Type: DXFImport
        """,
    )

    def _get_stator_dxf(self):
        """getter of stator_dxf"""
        return self._stator_dxf

    def _set_stator_dxf(self, value):
        """setter of stator_dxf"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "stator_dxf"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = DXFImport()
        check_var("stator_dxf", value, "DXFImport")
        self._stator_dxf = value

        if self._stator_dxf is not None:
            self._stator_dxf.parent = self

    stator_dxf = property(
        fget=_get_stator_dxf,
        fset=_set_stator_dxf,
        doc=u"""To use a dxf version of the rotor instead of build_geometry

        :Type: DXFImport
        """,
    )

    def _get_import_file(self):
        """getter of import_file"""
        return self._import_file

    def _set_import_file(self, value):
        """setter of import_file"""
        check_var("import_file", value, "str")
        self._import_file = value

    import_file = property(
        fget=_get_import_file,
        fset=_set_import_file,
        doc=u"""To import an existing femm file

        :Type: str
        """,
    )

    def _get_is_close_femm(self):
        """getter of is_close_femm"""
        return self._is_close_femm

    def _set_is_close_femm(self, value):
        """setter of is_close_femm"""
        check_var("is_close_femm", value, "bool")
        self._is_close_femm = value

    is_close_femm = property(
        fget=_get_is_close_femm,
        fset=_set_is_close_femm,
        doc=u"""To close femm automatically after the simulation

        :Type: bool
        """,
    )

    def _get_nb_worker(self):
        """getter of nb_worker"""
        return self._nb_worker

    def _set_nb_worker(self, value):
        """setter of nb_worker"""
        check_var("nb_worker", value, "int")
        self._nb_worker = value

    nb_worker = property(
        fget=_get_nb_worker,
        fset=_set_nb_worker,
        doc=u"""To run FEMM in parallel (the parallelization is on the time loop)

        :Type: int
        """,
    )

    def _get_Rag_enforced(self):
        """getter of Rag_enforced"""
        return self._Rag_enforced

    def _set_Rag_enforced(self, value):
        """setter of Rag_enforced"""
        check_var("Rag_enforced", value, "float")
        self._Rag_enforced = value

    Rag_enforced = property(
        fget=_get_Rag_enforced,
        fset=_set_Rag_enforced,
        doc=u"""To enforce a different radius value for air-gap outputs

        :Type: float
        """,
    )



from ....Methods.Machine.LamSlotWind import Lam_WindCheckError


class SquirrelCageCheckError(Lam_WindCheckError):
    """ """

    pass


class SCC_NotARotor(SquirrelCageCheckError):
    """ """

    pass


class SCC_WrongCond(SquirrelCageCheckError):
    """ """

    pass



def __getitem__(self, idx):
    """Method to behave like a list or a dict and iterate in the object"""
    if isinstance(idx, int) or isinstance(idx, slice):
        return self.output_list.__getitem__(idx)
    else:
        return self.xoutput_dict.__getitem__(idx)



from ...Classes.Material import Material
from ...Functions.Material.compare_material import compare_material


def get_material(obj):
    """
    Get the list of unique materials contained in a pyleecan object

    Parameters
    ----------
    obj : Pyleecan object

    Returns
    -------
    materials : list of unique materials contained in the object
    """
    materials = []
    for key, value in obj.as_dict().items():
        # Get an object attribute
        sub_obj = getattr(obj, key)

        # Add it if it's a Material
        if isinstance(sub_obj, Material):
            materials.append(sub_obj)

        # Check the sub object if it's a Pyleecan object
        elif isinstance(value, dict) and "__class__" in value:
            materials.extend(get_material(sub_obj))

    # Return a unique list
    for material in materials[:]:
        n_mat = 0
        for mat in materials:
            if compare_material(mat, material):
                n_mat += 1
                if n_mat > 1:  # material found twice in the list
                    materials.remove(material)
                    break

    return materials



def reverse(self):
    """Reverse the begin and end point of the Line

    Parameters
    ----------
    self : Arc3
        An Arc3 object

    Returns
    -------
    """

    end = self.end
    self.end = self.begin
    self.begin = end
    self.is_trigo_direction = not self.is_trigo_direction



# -*- coding: utf-8 -*-

from numpy import pi, exp


def comp_height_active(self):
    """Compute the height of the winding area

    Parameters
    ----------
    self : SlotW60
        A SlotW60 object

    Returns
    -------
    Hwind: float
        Height of the winding area [m]

    """

    [Z1, Z2, Z3, Z4, Z5, Z6, Z7, Z8, Z9, Z10, Z11] = self._comp_point_coordinate()

    # Compute the point in the tooth ref
    hsp = pi / self.Zs
    Z4t = Z4 * exp(1j * hsp)
    Zw4t = Z4t - self.H3 + 1j * ((self.W1 - self.W2) / 2 - self.W3)

    return abs(Zw4t) - abs(Z5)



# -*- coding: utf-8 -*-

from numpy import abs as np_abs, angle as np_angle, exp


def get_middle(self):
    """Return the point at the middle of the arc

    Parameters
    ----------
    self : Arc1
        An Arc1 object

    Returns
    -------
    Zmid: complex
        Complex coordinates of the middle of the Arc1
    """

    # We use the complex representation of the point
    z1 = self.begin
    z2 = self.end
    zc = self.get_center()

    # Geometric transformation : center is the origine, angle(begin) = 0
    Zstart = (z1 - zc) * exp(-1j * np_angle(z1 - zc))

    # Generation of the point by rotation
    alpha = self.get_angle()
    Zmid = Zstart * exp(1j * alpha / 2)

    # Geometric transformation : return to the main axis
    Zmid = Zmid * exp(1j * np_angle(z1 - zc)) + zc

    # Return (0,0) if the point is too close from 0
    if np_abs(Zmid) < 1e-6:
        Zmid = 0

    return Zmid



# -*- coding: utf-8 -*-

from numpy import pi, sign, sqrt

from ...Functions.FEMM.set_FEMM_circuit_prop import set_FEMM_circuit_prop
from ...Functions.Winding.find_wind_phase_color import get_phase_id
from ...Functions.FEMM.set_FEMM_wind_material import set_FEMM_wind_material


def create_FEMM_circuit_material(
    femm, circuits, label, is_eddies, lam, I, is_mmf, j_t0, materials
):
    """Set in FEMM circuits property

    Parameters
    ----------
    femm : FEMMHandler
        client to send command to a FEMM instance
    circuits: list
        list the name of all circuits
    label : str
        label of the surface
    sym : bool
        integer for symmetry
    is_eddies :
        1 to calculate eddy currents
    lam : LamSlotWind
        Lamination to set the winding
    I : ndarray
        Lamination currents waveforms
    is_mmf :
        1 to compute the lamination magnetomotive
        force / lamination magnetic field
    j_t0 :
        time step for winding current calculation@type integer
    materials :
        list of materials already created in FEMM

    Returns
    -------
    property, materials, circuits: tuple
        the property of the surface having label as surf.label (str),
        materials (list) and the circuits (list)
    """

    # Load parameter for readibility
    rho = lam.winding.conductor.cond_mat.elec.rho  # Resistivity at 20°C
    wind_mat = lam.winding.comp_connection_mat(lam.slot.Zs)
    Swire = lam.winding.conductor.comp_surface()

    # Decode the label
    st = label.split("_")
    Nrad_id = int(st[2][1:])  # zone radial coordinate
    Ntan_id = int(st[3][1:])  # zone tangential coordinate
    Zs_id = int(st[4][1:])  # Zone slot number coordinate
    # Get the phase value in the correct slot zone
    q_id = get_phase_id(wind_mat, Nrad_id, Ntan_id, Zs_id)
    s = sign(wind_mat[Nrad_id, Ntan_id, Zs_id, q_id])

    # circuit definition
    if "Rotor" in label:
        Clabel = "Circr" + str(q_id)
    else:
        Clabel = "Circs" + str(q_id)

    circuits = set_FEMM_circuit_prop(femm, circuits, Clabel, I=None)

    # definition of armature field current sources
    if "Rotor" in label:
        Jlabel = "Jr"
    else:
        Jlabel = "Js"
    if s == 1:
        cname = Jlabel + str(q_id) + "+"
    else:
        cname = Jlabel + str(q_id) + "-"

    # circuit definition
    circuits = set_FEMM_circuit_prop(femm, circuits, Clabel, I=None)

    # adding new current source material if necessary
    materials = set_FEMM_wind_material(
        femm,
        materials,
        cname=cname,
        Cduct=is_eddies * 1e-6 / rho,
        dwire=sqrt(4 * Swire / pi) * 1e-3,
    )

    return cname, materials, circuits



# -*- coding: utf-8 -*-
"""File generated according to PCondType11/gen_list.json
WARNING! All changes made in this file will be lost!
"""
from pyleecan.GUI.Dialog.DMachineSetup.SWindCond.PCondType11.Ui_PCondType11 import (
    Ui_PCondType11,
)


class Gen_PCondType11(Ui_PCondType11):
    def setupUi(self, PCondType11):
        """Abstract class to update the widget according to the csv doc"""
        Ui_PCondType11.setupUi(self, PCondType11)
        # Setup of in_Nwpc1_tan
        txt = self.tr(
            u"""cf schematics, stator winding number of preformed wires (strands) in parallel per coil along tangential (horizontal) direction"""
        )
        self.in_Nwpc1_tan.setWhatsThis(txt)
        self.in_Nwpc1_tan.setToolTip(txt)

        # Setup of si_Nwpc1_tan
        self.si_Nwpc1_tan.setMinimum(1)
        self.si_Nwpc1_tan.setMaximum(999999)
        txt = self.tr(
            u"""cf schematics, stator winding number of preformed wires (strands) in parallel per coil along tangential (horizontal) direction"""
        )
        self.si_Nwpc1_tan.setWhatsThis(txt)
        self.si_Nwpc1_tan.setToolTip(txt)

        # Setup of in_Nwpc1_rad
        txt = self.tr(
            u"""cf schematics, stator winding number of preformed wires (strands) in parallel per coil along radial (vertical) direction"""
        )
        self.in_Nwpc1_rad.setWhatsThis(txt)
        self.in_Nwpc1_rad.setToolTip(txt)

        # Setup of si_Nwpc1_rad
        self.si_Nwpc1_rad.setMinimum(1)
        self.si_Nwpc1_rad.setMaximum(999999)
        txt = self.tr(
            u"""cf schematics, stator winding number of preformed wires (strands) in parallel per coil along radial (vertical) direction"""
        )
        self.si_Nwpc1_rad.setWhatsThis(txt)
        self.si_Nwpc1_rad.setToolTip(txt)

        # Setup of in_Hwire
        txt = self.tr(u"""cf schematics, single wire height without insulation [m]""")
        self.in_Hwire.setWhatsThis(txt)
        self.in_Hwire.setToolTip(txt)

        # Setup of lf_Hwire
        self.lf_Hwire.validator().setBottom(0)
        txt = self.tr(u"""cf schematics, single wire height without insulation [m]""")
        self.lf_Hwire.setWhatsThis(txt)
        self.lf_Hwire.setToolTip(txt)

        # Setup of in_Wwire
        txt = self.tr(u"""cf schematics, single wire width without insulation [m]""")
        self.in_Wwire.setWhatsThis(txt)
        self.in_Wwire.setToolTip(txt)

        # Setup of lf_Wwire
        self.lf_Wwire.validator().setBottom(0)
        txt = self.tr(u"""cf schematics, single wire width without insulation [m]""")
        self.lf_Wwire.setWhatsThis(txt)
        self.lf_Wwire.setToolTip(txt)

        # Setup of in_Wins_wire
        txt = self.tr(
            u"""(advanced) cf schematics, winding strand insulation thickness [m]"""
        )
        self.in_Wins_wire.setWhatsThis(txt)
        self.in_Wins_wire.setToolTip(txt)

        # Setup of lf_Wins_wire
        self.lf_Wins_wire.validator().setBottom(0)
        txt = self.tr(
            u"""(advanced) cf schematics, winding strand insulation thickness [m]"""
        )
        self.lf_Wins_wire.setWhatsThis(txt)
        self.lf_Wins_wire.setToolTip(txt)



# -*- coding: utf-8 -*-

from numpy import linspace, zeros

from ....Classes.Segment import Segment
from ....Classes.Arc1 import Arc1
from ....Classes.SurfLine import SurfLine


def get_surface_active(self, alpha=0, delta=0):
    """Return the full active surface

    Parameters
    ----------
    self : SlotM15
        A SlotM15 object
    alpha : float
        float number for rotation (Default value = 0) [rad]
    delta : complex
        complex number for translation (Default value = 0)

    Returns
    -------
    surf_act: Surface
        Surface corresponding to the Active Area
    """

    # get the name of the lamination
    st = self.get_name_lam()

    point_dict = self._comp_point_coordinate()
    ZM0 = point_dict["ZM0"]
    ZM1 = point_dict["ZM1"]
    ZM2 = point_dict["ZM2"]
    ZM3 = point_dict["ZM3"]
    ZM4 = point_dict["ZM4"]

    curve_list = list()
    curve_list.append(Segment(ZM1, ZM2))
    if self.is_outwards():
        curve_list.append(Arc1(ZM2, ZM3, -self.Rtopm, is_trigo_direction=False))
    else:
        curve_list.append(Arc1(ZM2, ZM3, self.Rtopm))
    curve_list.append(Segment(ZM3, ZM4))
    curve_list.append(Arc1(ZM4, ZM1, -abs(ZM4), is_trigo_direction=False))

    Zmid = (abs(ZM1) + abs(ZM3)) / 2  # ref. point on x-axis with respective radius

    surface = SurfLine(
        line_list=curve_list, label="Wind_" + st + "_R0_T0_S0", point_ref=Zmid
    )

    # Apply transformation
    surface.rotate(alpha)
    surface.translate(delta)

    return surface



# -*- coding: utf-8 -*-


def remove_magnet(self):
    """Remove the magnet (set to None) of the Hole

    Parameters
    ----------
    self : HoleM53
        a HoleM53 object
    """

    self.magnet_0 = None
    self.magnet_1 = None



# -*- coding: utf-8 -*-
from ....Classes.Segment import Segment


def build_geometry(self):
    """Compute the curve (Line) needed to plot the Slot.
    The ending point of a curve is the starting point of the next curve in
    the list

    Parameters
    ----------
    self : SlotM16
        A SlotM16 object

    Returns
    -------
    curve_list: list
        A list of 11 Segment

    """

    point_dict = self._comp_point_coordinate()

    # Creation of curve
    curve_list = list()
    if self.H0 > 0:
        curve_list.append(Segment(point_dict["Z1"], point_dict["Z2"]))
    curve_list.append(Segment(point_dict["Z2"], point_dict["Z3"]))
    if self.H1 > 0:
        curve_list.append(Segment(point_dict["Z3"], point_dict["Z4"]))
    curve_list.append(Segment(point_dict["Z4"], point_dict["Z5"]))
    if self.H1 > 0:
        curve_list.append(Segment(point_dict["Z5"], point_dict["Z6"]))
    curve_list.append(Segment(point_dict["Z6"], point_dict["Z7"]))
    if self.H0 > 0:
        curve_list.append(Segment(point_dict["Z7"], point_dict["Z8"]))

    return curve_list



# -*- coding: utf-8 -*-

from numpy import abs as np_abs


def comp_radius(self):
    """Compute the radius of the arc

    Parameters
    ----------
    self : Arc3
        An Arc3 object

    Returns
    -------
    radius: float
        radius of the arc

    """

    self.check()

    z1 = self.begin
    z2 = self.end

    return np_abs(z1 - z2) / 2.0  # Radius of the arc



# -*- coding: utf-8 -*-


def get_boundary_condition(line, boundary_prop):
    """Returns the boundary name on a line that is used in FEA coupling

    Parameters
    ----------
    line :
        a line with a label

    Returns
    -------
    label : string
        boundary name
    """

    propname = ""
    for bound_label in boundary_prop:
        if bound_label in line.label:
            propname = boundary_prop[bound_label]

    return propname



# -*- coding: utf-8 -*-

# File generated according to SMachineType.ui
# WARNING! All changes made in this file will be lost!
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from pyleecan.GUI.Resources import pyleecan_rc


class Ui_SMachineType(object):
    def setupUi(self, SMachineType):
        if not SMachineType.objectName():
            SMachineType.setObjectName(u"SMachineType")
        SMachineType.resize(650, 550)
        SMachineType.setMinimumSize(QSize(650, 550))
        self.verticalLayout = QVBoxLayout(SMachineType)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.horizontalLayout_5 = QHBoxLayout()
        self.horizontalLayout_5.setObjectName(u"horizontalLayout_5")
        self.horizontalSpacer_2 = QSpacerItem(
            40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum
        )

        self.horizontalLayout_5.addItem(self.horizontalSpacer_2)

        self.in_machine_type = QLabel(SMachineType)
        self.in_machine_type.setObjectName(u"in_machine_type")

        self.horizontalLayout_5.addWidget(self.in_machine_type)

        self.c_type = QComboBox(SMachineType)
        self.c_type.addItem("")
        self.c_type.addItem("")
        self.c_type.addItem("")
        self.c_type.addItem("")
        self.c_type.addItem("")
        self.c_type.addItem("")
        self.c_type.addItem("")
        self.c_type.setObjectName(u"c_type")

        self.horizontalLayout_5.addWidget(self.c_type)

        self.horizontalSpacer_4 = QSpacerItem(
            40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum
        )

        self.horizontalLayout_5.addItem(self.horizontalSpacer_4)

        self.verticalLayout.addLayout(self.horizontalLayout_5)

        self.horizontalLayout_4 = QHBoxLayout()
        self.horizontalLayout_4.setObjectName(u"horizontalLayout_4")
        self.img_type_machine = QLabel(SMachineType)
        self.img_type_machine.setObjectName(u"img_type_machine")
        self.img_type_machine.setMinimumSize(QSize(300, 300))
        self.img_type_machine.setMaximumSize(QSize(400, 400))
        self.img_type_machine.setPixmap(
            QPixmap(u":/images/images/MachineSetup/MachineType/machine_SCIM.png")
        )
        self.img_type_machine.setScaledContents(True)

        self.horizontalLayout_4.addWidget(self.img_type_machine)

        self.verticalLayout.addLayout(self.horizontalLayout_4)

        self.txt_type_machine = QTextEdit(SMachineType)
        self.txt_type_machine.setObjectName(u"txt_type_machine")
        self.txt_type_machine.setMaximumSize(QSize(16777215, 30))
        self.txt_type_machine.setTextInteractionFlags(
            Qt.TextSelectableByKeyboard | Qt.TextSelectableByMouse
        )

        self.verticalLayout.addWidget(self.txt_type_machine)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.is_inner_rotor = QCheckBox(SMachineType)
        self.is_inner_rotor.setObjectName(u"is_inner_rotor")
        self.is_inner_rotor.setChecked(True)

        self.horizontalLayout.addWidget(self.is_inner_rotor)

        self.horizontalSpacer = QSpacerItem(
            40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum
        )

        self.horizontalLayout.addItem(self.horizontalSpacer)

        self.in_name = QLabel(SMachineType)
        self.in_name.setObjectName(u"in_name")

        self.horizontalLayout.addWidget(self.in_name)

        self.le_name = QLineEdit(SMachineType)
        self.le_name.setObjectName(u"le_name")
        self.le_name.setMinimumSize(QSize(200, 0))

        self.horizontalLayout.addWidget(self.le_name)

        self.horizontalSpacer_5 = QSpacerItem(
            40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum
        )

        self.horizontalLayout.addItem(self.horizontalSpacer_5)

        self.in_p = QLabel(SMachineType)
        self.in_p.setObjectName(u"in_p")

        self.horizontalLayout.addWidget(self.in_p)

        self.si_p = QSpinBox(SMachineType)
        self.si_p.setObjectName(u"si_p")
        self.si_p.setMinimum(1)
        self.si_p.setMaximum(999999)
        self.si_p.setValue(1)

        self.horizontalLayout.addWidget(self.si_p)

        self.verticalLayout.addLayout(self.horizontalLayout)

        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.horizontalSpacer_3 = QSpacerItem(
            40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum
        )

        self.horizontalLayout_2.addItem(self.horizontalSpacer_3)

        self.b_previous = QPushButton(SMachineType)
        self.b_previous.setObjectName(u"b_previous")
        self.b_previous.setEnabled(False)

        self.horizontalLayout_2.addWidget(self.b_previous)

        self.b_next = QPushButton(SMachineType)
        self.b_next.setObjectName(u"b_next")

        self.horizontalLayout_2.addWidget(self.b_next)

        self.verticalLayout.addLayout(self.horizontalLayout_2)

        QWidget.setTabOrder(self.si_p, self.b_next)

        self.retranslateUi(SMachineType)

        QMetaObject.connectSlotsByName(SMachineType)

    # setupUi

    def retranslateUi(self, SMachineType):
        SMachineType.setWindowTitle(
            QCoreApplication.translate("SMachineType", u"Form", None)
        )
        self.in_machine_type.setText(
            QCoreApplication.translate("SMachineType", u"Machine type :", None)
        )
        self.c_type.setItemText(
            0, QCoreApplication.translate("SMachineType", u"SCIM", None)
        )
        self.c_type.setItemText(
            1, QCoreApplication.translate("SMachineType", u"DFIM", None)
        )
        self.c_type.setItemText(
            2, QCoreApplication.translate("SMachineType", u"WRSM", None)
        )
        self.c_type.setItemText(
            3, QCoreApplication.translate("SMachineType", u"SPMSM", None)
        )
        self.c_type.setItemText(
            4, QCoreApplication.translate("SMachineType", u"SIPMSM", None)
        )
        self.c_type.setItemText(
            5, QCoreApplication.translate("SMachineType", u"IPMSM", None)
        )
        self.c_type.setItemText(
            6, QCoreApplication.translate("SMachineType", u"SyRM", None)
        )

        self.img_type_machine.setText("")
        # if QT_CONFIG(tooltip)
        self.is_inner_rotor.setToolTip(
            QCoreApplication.translate(
                "SMachineType",
                u"1 for internal rotor topology, 0 for external rotor",
                None,
            )
        )
        # endif // QT_CONFIG(tooltip)
        self.is_inner_rotor.setText(
            QCoreApplication.translate("SMachineType", u"is_inner_rotor", None)
        )
        self.in_name.setText(
            QCoreApplication.translate("SMachineType", u"Machine name :", None)
        )
        self.in_p.setText(QCoreApplication.translate("SMachineType", u"p :", None))
        self.b_previous.setText(
            QCoreApplication.translate("SMachineType", u"Previous", None)
        )
        self.b_next.setText(QCoreApplication.translate("SMachineType", u"Next", None))

    # retranslateUi



# Default number of point added for the discrtization of an arc
ARC_NPOINT_D = 99
# Default number of point added for the discrtization of a line
LINE_NPOINT_D = 0


class MachineCheckError(Exception):
    """ """

    pass



# -*- coding: utf-8 -*-


def getattr_recursive(obj, attr_list):
    """
    Function to recursively get an attribute.

    Parameters
    ----------
    obj : object
        An object

    attr_list: list
        A list of strings that is the attribute 'tree'

    Returns
    -------
    attr: attribute
        An attribute

    """

    if len(attr_list) > 1:
        return getattr_recursive(getattr(obj, attr_list[0]), attr_list[1:])
    else:
        return getattr(obj, attr_list[0])



# -*- coding: utf-8 -*-


def get_center(self):
    """Return the center point of the arc

    Parameters
    ----------
    self : Arc2
        An Arc2 object

    Returns
    -------
    center: complex
        center point of the Arc2
    """

    return self.center



from ....Methods.Slot.Hole import MagnetIndexError


def get_magnet_by_id(self, index):
    """Return the magnet at the corresponding index
    returns None is the magnet is None
    Raise error if the hole doesn't have a magnet for the corresponding index

    Parameters
    ----------
    self : Hole
        A Hole object
    index : int
        Index of the magnet to return
    Returns
    -------
    magnet : Magnet
        Magnet object (or None)
    """

    label = "magnet_" + str(index)
    if hasattr(self, label):
        return getattr(self, label)
    elif hasattr(self, "magnet_dict") and label in self.magnet_dict:
        return self.magnet_dict[label]
    else:
        raise MagnetIndexError(
            "Hole of type " + str(type(self)) + " has no magnet " + label
        )



# -*- coding: utf-8 -*-

# File generated according to PCondType12.ui
# WARNING! All changes made in this file will be lost!
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from ......GUI.Tools.FloatEdit import FloatEdit
from ......GUI.Dialog.DMachineSetup.SWindCond.WCondOut.WCondOut import WCondOut

from pyleecan.GUI.Resources import pyleecan_rc


class Ui_PCondType12(object):
    def setupUi(self, PCondType12):
        if not PCondType12.objectName():
            PCondType12.setObjectName(u"PCondType12")
        PCondType12.resize(1189, 672)
        self.horizontalLayout = QHBoxLayout(PCondType12)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.img_cond = QLabel(PCondType12)
        self.img_cond.setObjectName(u"img_cond")
        self.img_cond.setMinimumSize(QSize(0, 0))
        self.img_cond.setMaximumSize(QSize(16777215, 16777215))
        self.img_cond.setPixmap(
            QPixmap(u":/images/images/MachineSetup/WindParam/CondType12.png")
        )
        self.img_cond.setScaledContents(True)

        self.horizontalLayout.addWidget(self.img_cond)

        self.widget = QWidget(PCondType12)
        self.widget.setObjectName(u"widget")
        self.widget.setMinimumSize(QSize(250, 0))
        self.widget.setMaximumSize(QSize(250, 16777215))
        self.verticalLayout = QVBoxLayout(self.widget)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.gridLayout = QGridLayout()
        self.gridLayout.setObjectName(u"gridLayout")
        self.in_Nwpc1 = QLabel(self.widget)
        self.in_Nwpc1.setObjectName(u"in_Nwpc1")
        self.in_Nwpc1.setMinimumSize(QSize(70, 0))

        self.gridLayout.addWidget(self.in_Nwpc1, 0, 0, 1, 1)

        self.si_Nwpc1 = QSpinBox(self.widget)
        self.si_Nwpc1.setObjectName(u"si_Nwpc1")
        self.si_Nwpc1.setMinimumSize(QSize(0, 0))
        self.si_Nwpc1.setValue(99)

        self.gridLayout.addWidget(self.si_Nwpc1, 0, 1, 1, 1)

        self.in_Wwire = QLabel(self.widget)
        self.in_Wwire.setObjectName(u"in_Wwire")
        self.in_Wwire.setMinimumSize(QSize(70, 0))

        self.gridLayout.addWidget(self.in_Wwire, 1, 0, 1, 1)

        self.lf_Wwire = FloatEdit(self.widget)
        self.lf_Wwire.setObjectName(u"lf_Wwire")
        self.lf_Wwire.setMinimumSize(QSize(0, 0))
        self.lf_Wwire.setMaximumSize(QSize(16777215, 16777215))

        self.gridLayout.addWidget(self.lf_Wwire, 1, 1, 1, 1)

        self.unit_Wwire = QLabel(self.widget)
        self.unit_Wwire.setObjectName(u"unit_Wwire")
        self.unit_Wwire.setMinimumSize(QSize(0, 0))

        self.gridLayout.addWidget(self.unit_Wwire, 1, 2, 1, 1)

        self.in_Wins_wire = QLabel(self.widget)
        self.in_Wins_wire.setObjectName(u"in_Wins_wire")
        self.in_Wins_wire.setMinimumSize(QSize(40, 0))

        self.gridLayout.addWidget(self.in_Wins_wire, 2, 0, 1, 1)

        self.lf_Wins_wire = FloatEdit(self.widget)
        self.lf_Wins_wire.setObjectName(u"lf_Wins_wire")
        self.lf_Wins_wire.setMinimumSize(QSize(50, 0))
        self.lf_Wins_wire.setMaximumSize(QSize(16777215, 16777215))

        self.gridLayout.addWidget(self.lf_Wins_wire, 2, 1, 1, 1)

        self.unit_Wins_wire = QLabel(self.widget)
        self.unit_Wins_wire.setObjectName(u"unit_Wins_wire")
        self.unit_Wins_wire.setMinimumSize(QSize(0, 0))

        self.gridLayout.addWidget(self.unit_Wins_wire, 2, 2, 1, 1)

        self.in_Wins_cond = QLabel(self.widget)
        self.in_Wins_cond.setObjectName(u"in_Wins_cond")

        self.gridLayout.addWidget(self.in_Wins_cond, 3, 0, 1, 1)

        self.lf_Wins_cond = FloatEdit(self.widget)
        self.lf_Wins_cond.setObjectName(u"lf_Wins_cond")
        self.lf_Wins_cond.setMinimumSize(QSize(50, 0))
        self.lf_Wins_cond.setMaximumSize(QSize(16777215, 16777215))

        self.gridLayout.addWidget(self.lf_Wins_cond, 3, 1, 1, 1)

        self.unit_Wins_cond = QLabel(self.widget)
        self.unit_Wins_cond.setObjectName(u"unit_Wins_cond")

        self.gridLayout.addWidget(self.unit_Wins_cond, 3, 2, 1, 1)

        self.in_Lewout = QLabel(self.widget)
        self.in_Lewout.setObjectName(u"in_Lewout")
        self.in_Lewout.setMinimumSize(QSize(40, 0))

        self.gridLayout.addWidget(self.in_Lewout, 4, 0, 1, 1)

        self.lf_Lewout = FloatEdit(self.widget)
        self.lf_Lewout.setObjectName(u"lf_Lewout")
        self.lf_Lewout.setMinimumSize(QSize(50, 0))
        self.lf_Lewout.setMaximumSize(QSize(16777215, 16777215))

        self.gridLayout.addWidget(self.lf_Lewout, 4, 1, 1, 1)

        self.unit_Lewout = QLabel(self.widget)
        self.unit_Lewout.setObjectName(u"unit_Lewout")
        self.unit_Lewout.setMinimumSize(QSize(0, 0))

        self.gridLayout.addWidget(self.unit_Lewout, 4, 2, 1, 1)

        self.verticalLayout.addLayout(self.gridLayout)

        self.verticalSpacer = QSpacerItem(
            20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding
        )

        self.verticalLayout.addItem(self.verticalSpacer)

        self.w_out = WCondOut(self.widget)
        self.w_out.setObjectName(u"w_out")

        self.verticalLayout.addWidget(self.w_out)

        self.horizontalLayout.addWidget(self.widget)

        QWidget.setTabOrder(self.si_Nwpc1, self.lf_Wwire)
        QWidget.setTabOrder(self.lf_Wwire, self.lf_Wins_wire)

        self.retranslateUi(PCondType12)

        QMetaObject.connectSlotsByName(PCondType12)

    # setupUi

    def retranslateUi(self, PCondType12):
        PCondType12.setWindowTitle(
            QCoreApplication.translate("PCondType12", u"Form", None)
        )
        self.img_cond.setText("")
        self.in_Nwpc1.setText(QCoreApplication.translate("PCondType12", u"Nwppc", None))
        self.in_Wwire.setText(QCoreApplication.translate("PCondType12", u"Wwire", None))
        self.unit_Wwire.setText(QCoreApplication.translate("PCondType12", u"m", None))
        self.in_Wins_wire.setText(
            QCoreApplication.translate("PCondType12", u"Wins_wire", None)
        )
        self.unit_Wins_wire.setText(
            QCoreApplication.translate("PCondType12", u"m", None)
        )
        self.in_Wins_cond.setText(
            QCoreApplication.translate("PCondType12", u"Wins_cond", None)
        )
        self.unit_Wins_cond.setText(
            QCoreApplication.translate("PCondType12", u"m", None)
        )
        # if QT_CONFIG(tooltip)
        self.in_Lewout.setToolTip(
            QCoreApplication.translate(
                "PCondType12", u"End-winding length on one side for a half-turn", None
            )
        )
        # endif // QT_CONFIG(tooltip)
        # if QT_CONFIG(whatsthis)
        self.in_Lewout.setWhatsThis(
            QCoreApplication.translate(
                "PCondType12", u"End-winding length on one side for a half-turn", None
            )
        )
        # endif // QT_CONFIG(whatsthis)
        self.in_Lewout.setText(
            QCoreApplication.translate("PCondType12", u"Lewout", None)
        )
        # if QT_CONFIG(tooltip)
        self.lf_Lewout.setToolTip(
            QCoreApplication.translate(
                "PCondType12", u"End-winding length on one side for a half-turn", None
            )
        )
        # endif // QT_CONFIG(tooltip)
        # if QT_CONFIG(whatsthis)
        self.lf_Lewout.setWhatsThis(
            QCoreApplication.translate(
                "PCondType12", u"End-winding length on one side for a half-turn", None
            )
        )
        # endif // QT_CONFIG(whatsthis)
        # if QT_CONFIG(tooltip)
        self.unit_Lewout.setToolTip(
            QCoreApplication.translate(
                "PCondType12", u"End-winding length on one side for a half-turn", None
            )
        )
        # endif // QT_CONFIG(tooltip)
        # if QT_CONFIG(whatsthis)
        self.unit_Lewout.setWhatsThis(
            QCoreApplication.translate(
                "PCondType12", u"End-winding length on one side for a half-turn", None
            )
        )
        # endif // QT_CONFIG(whatsthis)
        self.unit_Lewout.setText(QCoreApplication.translate("PCondType12", u"m", None))

    # retranslateUi



from ....Classes.Simulation import Simulation
from ....Methods.Simulation.Input import InputError


def comp_felec(self):
    """Compute the electrical frequency

    Parameters
    ----------
    felec : float
        Electrical frequency [Hz]
    """

    if self.OP.felec is not None:
        return self.OP.felec  # TODO maybe add some checks?
    elif self.OP.N0 is not None:
        # Get the phase number for verifications
        if self.parent is None:
            raise InputError(
                "ERROR: InputFlux object should be inside a Simulation object"
            )
        # get the simulation
        if isinstance(self.parent, Simulation):
            simu = self.parent
        elif isinstance(self.parent.parent, Simulation):
            simu = self.parent.parent
        else:
            raise InputError("Cannot find InputFlux simulation.")

        zp = simu.machine.stator.get_pole_pair_number()
        zp = zp if zp is not None else 1

        return self.OP.N0 * zp / 60
    else:
        raise InputError(
            "ERROR: InputFlux object can't have OP.felec and OP.N0 at None"
        )



# -*- coding: utf-8 -*-
from SciDataTool import DataFreq, Data1D
from numpy import newaxis, abs, sqrt, stack


def comp_loss_density(self, meshsolution):
    """
    Compute the losses density (per kg) according to the following model equation:
        Loss = C0*f*B^C1 + C2*(f*B)^C3 + C4*(f*B)^C4

    Parameters
    ----------
    self : LossModelBertotti
        a LossModelBertotti object
    field : DataND
        a DataND object that contains the flux density values

    Returns
    -------
    loss_density: DataND
        a DataND object of the normalized losses
    """
    # get the parameters
    Coeff = list(
        [self.k_hy, self.alpha_hy, self.k_ed, self.alpha_ed, self.k_ex, self.alpha_ex]
    )
    F_REF = self.F_REF
    B_REF = self.B_REF

    # filter needed mesh group
    sol = meshsolution.get_solution(label="B")

    # TODO Calculate principle axes and transform for exponentials other than 2
    # TODO maybe use rad. and tan. comp. as intermediate solution

    # loop over field components
    HY, ED, EX = None, None, None
    for component in sol.field.components.values():
        axes_names = ["freqs" if x.name == "time" else x.name for x in component.axes]

        # TODO add filter function to limit max. order of harmonics
        mag_dict = component.get_magnitude_along(*axes_names)
        symbol = component.symbol

        # TODO better data check (axis size, ...)
        freqs = mag_dict["freqs"]
        # freqs[freqs<0] = 0 # to only regard positive freqs
        k = 1  # 1 / sqrt(2)
        f_norm = abs(freqs[:, newaxis] / F_REF)
        B_norm = k * mag_dict[symbol] / B_REF
        # factor 1 / sqrt(2) to account for SciDataTool FFT of double sided spectrum
        # TODO is this factor also true for powers other than 2 ?

        HY_ = Coeff[0] * f_norm * B_norm ** Coeff[1]
        ED_ = Coeff[2] * (f_norm * B_norm) ** Coeff[3]
        EX_ = Coeff[4] * (f_norm * B_norm) ** Coeff[5]

        HY = HY_ if HY is None else HY + HY_
        ED = ED_ if ED is None else ED + ED_
        EX = EX_ if EX is None else EX + EX_

    loss = HY + ED + EX

    # setup loss density data
    Freq = Data1D(name="freqs", unit="", values=freqs)
    axes = [Freq if x.name == "time" else x for x in component.axes]

    loss_density = DataFreq(
        name="Loss Density", unit="W/kg", symbol="LossDens", axes=axes, values=loss
    )

    # setup loss density components data
    Comps = Data1D(
        name="Components",
        unit="W/kg",
        values=["Hysteresis", "Eddy", "Excess"],
        is_components=True,
    )
    axes = [axis.copy() for axis in axes]
    axes.append(Comps)

    comps_data = stack([HY, ED, EX], axis=len(axes) - 1)

    loss_density_comps = DataFreq(
        name="Loss Density Components",
        unit="W/kg",
        symbol="LossDensComps",
        axes=axes,
        values=comps_data,
    )

    return loss_density, loss_density_comps



# -*- coding: utf-8 -*-

from ....Classes.OutGeoLam import OutGeoLam


def comp_output_geo(self):
    """Compute the main geometry output

    Parameters
    ----------
    self : Lamination
        A Lamination object

    Returns
    -------
    output: OutGeoLam
        Main geometry output of the lamintion

    """

    output = OutGeoLam()
    output.name_phase = self.get_name_phase()
    output.BH_curve = self.mat_type.mag.get_BH()

    return output



# -*- coding: utf-8 -*-

from ....Methods.Simulation.StructElmer import _execute


def solve_FEA(self, output):
    """Solve the FEA simulation

    Parameters
    ----------


    Return
    ------


    """
    cwd = self.get_path_save_fea(output)  # current working dir

    logger = self.get_logger()

    for info in _execute("ElmerSolver", cwd, logger):
        print(info, end="")



# -*-- coding: utf-8 -*
def discretize(self, Npoint=-1):
    """Returns the discretize version of the SurfRing

    Parameters
    ----------
    self: SurfRing
        A SurfRing object
    Npoint : int
        Number of point on each line (Default value = -1 => use the line default discretization)

    Returns
    -------
    point_list : list
        List of complex coordinates
    """
    # check if the SurfRing is correct
    self.check()
    # getting lines that delimit the SurfLine

    point_list = self.out_surf.discretize(Npoint=Npoint)
    point_list.extend(self.in_surf.discretize(Npoint=Npoint))
    return point_list



# -*- coding: utf-8 -*-
from ....Classes.Arc1 import Arc1
from ....Classes.SurfLine import SurfLine


def get_surface(self):
    """Returns the surface delimiting the slot

    Parameters
    ----------
    self : Slot
        A Slot object

    Returns
    -------
    surface: SurfLine
        A SurfLine object representing the slot

    """
    Rbo = self.get_Rbo()
    curve_list = self.build_geometry()
    Zbegin = curve_list[-1].get_end()
    Zend = curve_list[0].get_begin()
    curve_list.append(Arc1(Zbegin, Zend, -Rbo, is_trigo_direction=False))

    return SurfLine(line_list=curve_list, label="Slot")



from logging import Filter, DEBUG, INFO, WARNING, ERROR, CRITICAL
from .definitions import USER_DIR

"""
File containing every Pyleecan loggers.
The choice between LOGGING_CONFIG_CONSOLE and LOGGING_CONFIG_FILE is made in pyleecan.__init__.py
"""

# Default level in every loggers
DEFAULT_LOG_NAME = "Pyleecan"
GUI_LOG_NAME = "Pyleecan.GUI"
SUB_LOG_LIST = ["Machine", "Electrical", "Magnetics", "Force", "Structural", "Loss"]

CONSOLE_LEVEL = INFO
CONSOLE_FORMAT = "[%(asctime)s] %(message)s"
CONSOLE_DATE_FORMAT = "%H:%M:%S"

FILE_LEVEL = DEBUG
FILE_FORMAT = "%(asctime)s-%(levelname)7s: %(message)s [%(name)s]"
FILE_DATE_FORMAT = "%Y-%m-%d %H:%M:%S"


def gen_logger_config_dict(logger_name):
    log_config_dict = {
        "version": 1,
        # Define loggers
        "loggers": {
            logger_name: {  # Default logger
                "level": "DEBUG",
                "propagate": False,
                "handlers": [
                    "console_handler",
                    "file_handler",
                ],  # Different handler enable to use differents formats, and to handle different stream (stdout, stderr,...)
            },
        },
        # Handlers definition
        "handlers": {
            "console_handler": {
                "level": CONSOLE_LEVEL,
                "formatter": "console_formatter",
                "class": "logging.StreamHandler",
                "stream": "ext://sys.stdout",
            },
            "file_handler": {  # <------- FileHandler
                "level": FILE_LEVEL,
                "formatter": "file_formatter",
                "class": "logging.FileHandler",
                "filename": USER_DIR + "/Pyleecan.log",  # log file
                "mode": "a",
            },
        },
        # Define formatters
        "formatters": {
            "console_formatter": {
                "format": CONSOLE_FORMAT,
                "datefmt": CONSOLE_DATE_FORMAT,
            },
            "file_formatter": {"format": FILE_FORMAT, "datefmt": FILE_DATE_FORMAT},
        },
        "disable_existing_loggers": False,
    }

    for log in SUB_LOG_LIST:
        log_config_dict["loggers"][logger_name + "." + log] = {
            "level": "DEBUG",
            "propagate": True,
            "handlers": [],
        }
    return log_config_dict



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Simulation/InputCurrent.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Simulation/InputCurrent
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .Input import Input

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Simulation.InputCurrent.gen_input import gen_input
except ImportError as error:
    gen_input = error

try:
    from ..Methods.Simulation.InputCurrent.set_Id_Iq import set_Id_Iq
except ImportError as error:
    set_Id_Iq = error

try:
    from ..Methods.Simulation.InputCurrent.set_OP_from_array import set_OP_from_array
except ImportError as error:
    set_OP_from_array = error


from ..Classes.ImportMatrixVal import ImportMatrixVal
from numpy import ndarray
from numpy import array, array_equal
from ._check import InitUnKnowClassError
from .ImportMatrix import ImportMatrix
from .Import import Import


class InputCurrent(Input):
    """Input to skip the electrical module and start with the magnetic one"""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Simulation.InputCurrent.gen_input
    if isinstance(gen_input, ImportError):
        gen_input = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use InputCurrent method gen_input: " + str(gen_input)
                )
            )
        )
    else:
        gen_input = gen_input
    # cf Methods.Simulation.InputCurrent.set_Id_Iq
    if isinstance(set_Id_Iq, ImportError):
        set_Id_Iq = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use InputCurrent method set_Id_Iq: " + str(set_Id_Iq)
                )
            )
        )
    else:
        set_Id_Iq = set_Id_Iq
    # cf Methods.Simulation.InputCurrent.set_OP_from_array
    if isinstance(set_OP_from_array, ImportError):
        set_OP_from_array = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use InputCurrent method set_OP_from_array: "
                    + str(set_OP_from_array)
                )
            )
        )
    else:
        set_OP_from_array = set_OP_from_array
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        Is=None,
        Ir=None,
        angle_rotor=None,
        rot_dir=None,
        angle_rotor_initial=0,
        Tem_av_ref=None,
        Id_ref=None,
        Iq_ref=None,
        felec=None,
        time=None,
        angle=None,
        Nt_tot=2048,
        Nrev=1,
        Na_tot=2048,
        N0=None,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "Is" in list(init_dict.keys()):
                Is = init_dict["Is"]
            if "Ir" in list(init_dict.keys()):
                Ir = init_dict["Ir"]
            if "angle_rotor" in list(init_dict.keys()):
                angle_rotor = init_dict["angle_rotor"]
            if "rot_dir" in list(init_dict.keys()):
                rot_dir = init_dict["rot_dir"]
            if "angle_rotor_initial" in list(init_dict.keys()):
                angle_rotor_initial = init_dict["angle_rotor_initial"]
            if "Tem_av_ref" in list(init_dict.keys()):
                Tem_av_ref = init_dict["Tem_av_ref"]
            if "Id_ref" in list(init_dict.keys()):
                Id_ref = init_dict["Id_ref"]
            if "Iq_ref" in list(init_dict.keys()):
                Iq_ref = init_dict["Iq_ref"]
            if "felec" in list(init_dict.keys()):
                felec = init_dict["felec"]
            if "time" in list(init_dict.keys()):
                time = init_dict["time"]
            if "angle" in list(init_dict.keys()):
                angle = init_dict["angle"]
            if "Nt_tot" in list(init_dict.keys()):
                Nt_tot = init_dict["Nt_tot"]
            if "Nrev" in list(init_dict.keys()):
                Nrev = init_dict["Nrev"]
            if "Na_tot" in list(init_dict.keys()):
                Na_tot = init_dict["Na_tot"]
            if "N0" in list(init_dict.keys()):
                N0 = init_dict["N0"]
        # Set the properties (value check and convertion are done in setter)
        self.Is = Is
        self.Ir = Ir
        self.angle_rotor = angle_rotor
        self.rot_dir = rot_dir
        self.angle_rotor_initial = angle_rotor_initial
        self.Tem_av_ref = Tem_av_ref
        self.Id_ref = Id_ref
        self.Iq_ref = Iq_ref
        self.felec = felec
        # Call Input init
        super(InputCurrent, self).__init__(
            time=time, angle=angle, Nt_tot=Nt_tot, Nrev=Nrev, Na_tot=Na_tot, N0=N0
        )
        # The class is frozen (in Input init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        InputCurrent_str = ""
        # Get the properties inherited from Input
        InputCurrent_str += super(InputCurrent, self).__str__()
        if self.Is is not None:
            tmp = self.Is.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            InputCurrent_str += "Is = " + tmp
        else:
            InputCurrent_str += "Is = None" + linesep + linesep
        if self.Ir is not None:
            tmp = self.Ir.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            InputCurrent_str += "Ir = " + tmp
        else:
            InputCurrent_str += "Ir = None" + linesep + linesep
        if self.angle_rotor is not None:
            tmp = (
                self.angle_rotor.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            )
            InputCurrent_str += "angle_rotor = " + tmp
        else:
            InputCurrent_str += "angle_rotor = None" + linesep + linesep
        InputCurrent_str += "rot_dir = " + str(self.rot_dir) + linesep
        InputCurrent_str += (
            "angle_rotor_initial = " + str(self.angle_rotor_initial) + linesep
        )
        InputCurrent_str += "Tem_av_ref = " + str(self.Tem_av_ref) + linesep
        InputCurrent_str += "Id_ref = " + str(self.Id_ref) + linesep
        InputCurrent_str += "Iq_ref = " + str(self.Iq_ref) + linesep
        InputCurrent_str += "felec = " + str(self.felec) + linesep
        return InputCurrent_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from Input
        if not super(InputCurrent, self).__eq__(other):
            return False
        if other.Is != self.Is:
            return False
        if other.Ir != self.Ir:
            return False
        if other.angle_rotor != self.angle_rotor:
            return False
        if other.rot_dir != self.rot_dir:
            return False
        if other.angle_rotor_initial != self.angle_rotor_initial:
            return False
        if other.Tem_av_ref != self.Tem_av_ref:
            return False
        if other.Id_ref != self.Id_ref:
            return False
        if other.Iq_ref != self.Iq_ref:
            return False
        if other.felec != self.felec:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from Input
        diff_list.extend(super(InputCurrent, self).compare(other, name=name))
        if (other.Is is None and self.Is is not None) or (
            other.Is is not None and self.Is is None
        ):
            diff_list.append(name + ".Is None mismatch")
        elif self.Is is not None:
            diff_list.extend(self.Is.compare(other.Is, name=name + ".Is"))
        if (other.Ir is None and self.Ir is not None) or (
            other.Ir is not None and self.Ir is None
        ):
            diff_list.append(name + ".Ir None mismatch")
        elif self.Ir is not None:
            diff_list.extend(self.Ir.compare(other.Ir, name=name + ".Ir"))
        if (other.angle_rotor is None and self.angle_rotor is not None) or (
            other.angle_rotor is not None and self.angle_rotor is None
        ):
            diff_list.append(name + ".angle_rotor None mismatch")
        elif self.angle_rotor is not None:
            diff_list.extend(
                self.angle_rotor.compare(other.angle_rotor, name=name + ".angle_rotor")
            )
        if other._rot_dir != self._rot_dir:
            diff_list.append(name + ".rot_dir")
        if other._angle_rotor_initial != self._angle_rotor_initial:
            diff_list.append(name + ".angle_rotor_initial")
        if other._Tem_av_ref != self._Tem_av_ref:
            diff_list.append(name + ".Tem_av_ref")
        if other._Id_ref != self._Id_ref:
            diff_list.append(name + ".Id_ref")
        if other._Iq_ref != self._Iq_ref:
            diff_list.append(name + ".Iq_ref")
        if other._felec != self._felec:
            diff_list.append(name + ".felec")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from Input
        S += super(InputCurrent, self).__sizeof__()
        S += getsizeof(self.Is)
        S += getsizeof(self.Ir)
        S += getsizeof(self.angle_rotor)
        S += getsizeof(self.rot_dir)
        S += getsizeof(self.angle_rotor_initial)
        S += getsizeof(self.Tem_av_ref)
        S += getsizeof(self.Id_ref)
        S += getsizeof(self.Iq_ref)
        S += getsizeof(self.felec)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from Input
        InputCurrent_dict = super(InputCurrent, self).as_dict(**kwargs)
        if self.Is is None:
            InputCurrent_dict["Is"] = None
        else:
            InputCurrent_dict["Is"] = self.Is.as_dict(**kwargs)
        if self.Ir is None:
            InputCurrent_dict["Ir"] = None
        else:
            InputCurrent_dict["Ir"] = self.Ir.as_dict(**kwargs)
        if self.angle_rotor is None:
            InputCurrent_dict["angle_rotor"] = None
        else:
            InputCurrent_dict["angle_rotor"] = self.angle_rotor.as_dict(**kwargs)
        InputCurrent_dict["rot_dir"] = self.rot_dir
        InputCurrent_dict["angle_rotor_initial"] = self.angle_rotor_initial
        InputCurrent_dict["Tem_av_ref"] = self.Tem_av_ref
        InputCurrent_dict["Id_ref"] = self.Id_ref
        InputCurrent_dict["Iq_ref"] = self.Iq_ref
        InputCurrent_dict["felec"] = self.felec
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        InputCurrent_dict["__class__"] = "InputCurrent"
        return InputCurrent_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        if self.Is is not None:
            self.Is._set_None()
        if self.Ir is not None:
            self.Ir._set_None()
        if self.angle_rotor is not None:
            self.angle_rotor._set_None()
        self.rot_dir = None
        self.angle_rotor_initial = None
        self.Tem_av_ref = None
        self.Id_ref = None
        self.Iq_ref = None
        self.felec = None
        # Set to None the properties inherited from Input
        super(InputCurrent, self)._set_None()

    def _get_Is(self):
        """getter of Is"""
        return self._Is

    def _set_Is(self, value):
        """setter of Is"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, ndarray):
            value = ImportMatrixVal(value=value)
        elif isinstance(value, list):
            value = ImportMatrixVal(value=array(value))
        elif value == -1:
            value = ImportMatrix()
        elif isinstance(value, dict):
            class_obj = import_class("pyleecan.Classes", value.get("__class__"), "Is")
            value = class_obj(init_dict=value)
        check_var("Is", value, "ImportMatrix")
        self._Is = value

        if self._Is is not None:
            self._Is.parent = self

    Is = property(
        fget=_get_Is,
        fset=_set_Is,
        doc=u"""Stator currents as a function of time (each column correspond to one phase) to import

        :Type: ImportMatrix
        """,
    )

    def _get_Ir(self):
        """getter of Ir"""
        return self._Ir

    def _set_Ir(self, value):
        """setter of Ir"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, ndarray):
            value = ImportMatrixVal(value=value)
        elif isinstance(value, list):
            value = ImportMatrixVal(value=array(value))
        elif value == -1:
            value = ImportMatrix()
        elif isinstance(value, dict):
            class_obj = import_class("pyleecan.Classes", value.get("__class__"), "Ir")
            value = class_obj(init_dict=value)
        check_var("Ir", value, "ImportMatrix")
        self._Ir = value

        if self._Ir is not None:
            self._Ir.parent = self

    Ir = property(
        fget=_get_Ir,
        fset=_set_Ir,
        doc=u"""Rotor currents as a function of time (each column correspond to one phase) to import

        :Type: ImportMatrix
        """,
    )

    def _get_angle_rotor(self):
        """getter of angle_rotor"""
        return self._angle_rotor

    def _set_angle_rotor(self, value):
        """setter of angle_rotor"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "angle_rotor"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = Import()
        check_var("angle_rotor", value, "Import")
        self._angle_rotor = value

        if self._angle_rotor is not None:
            self._angle_rotor.parent = self

    angle_rotor = property(
        fget=_get_angle_rotor,
        fset=_set_angle_rotor,
        doc=u"""Rotor angular position as a function of time (if None computed according to Nr) to import

        :Type: Import
        """,
    )

    def _get_rot_dir(self):
        """getter of rot_dir"""
        return self._rot_dir

    def _set_rot_dir(self, value):
        """setter of rot_dir"""
        check_var("rot_dir", value, "float", Vmin=-1, Vmax=1)
        self._rot_dir = value

    rot_dir = property(
        fget=_get_rot_dir,
        fset=_set_rot_dir,
        doc=u"""Rotation direction of the rotor 1 trigo, -1 clockwise

        :Type: float
        :min: -1
        :max: 1
        """,
    )

    def _get_angle_rotor_initial(self):
        """getter of angle_rotor_initial"""
        return self._angle_rotor_initial

    def _set_angle_rotor_initial(self, value):
        """setter of angle_rotor_initial"""
        check_var("angle_rotor_initial", value, "float")
        self._angle_rotor_initial = value

    angle_rotor_initial = property(
        fget=_get_angle_rotor_initial,
        fset=_set_angle_rotor_initial,
        doc=u"""Initial angular position of the rotor at t=0

        :Type: float
        """,
    )

    def _get_Tem_av_ref(self):
        """getter of Tem_av_ref"""
        return self._Tem_av_ref

    def _set_Tem_av_ref(self, value):
        """setter of Tem_av_ref"""
        check_var("Tem_av_ref", value, "float")
        self._Tem_av_ref = value

    Tem_av_ref = property(
        fget=_get_Tem_av_ref,
        fset=_set_Tem_av_ref,
        doc=u"""Theorical Average Electromagnetic torque

        :Type: float
        """,
    )

    def _get_Id_ref(self):
        """getter of Id_ref"""
        return self._Id_ref

    def _set_Id_ref(self, value):
        """setter of Id_ref"""
        check_var("Id_ref", value, "float")
        self._Id_ref = value

    Id_ref = property(
        fget=_get_Id_ref,
        fset=_set_Id_ref,
        doc=u"""d-axis current RMS magnitude

        :Type: float
        """,
    )

    def _get_Iq_ref(self):
        """getter of Iq_ref"""
        return self._Iq_ref

    def _set_Iq_ref(self, value):
        """setter of Iq_ref"""
        check_var("Iq_ref", value, "float")
        self._Iq_ref = value

    Iq_ref = property(
        fget=_get_Iq_ref,
        fset=_set_Iq_ref,
        doc=u"""q-axis current RMS magnitude

        :Type: float
        """,
    )

    def _get_felec(self):
        """getter of felec"""
        return self._felec

    def _set_felec(self, value):
        """setter of felec"""
        check_var("felec", value, "float")
        self._felec = value

    felec = property(
        fget=_get_felec,
        fset=_set_felec,
        doc=u"""electrical frequency

        :Type: float
        """,
    )



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Output/OutStruct.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Output/OutStruct
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from ._frozen import FrozenClass

from ._check import InitUnKnowClassError
from .MeshSolution import MeshSolution


class OutStruct(FrozenClass):
    """Gather the structural module outputs"""

    VERSION = 1

    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        Time=None,
        Angle=None,
        Nt_tot=None,
        Na_tot=None,
        logger_name="Pyleecan.Structural",
        Yr=None,
        Vr=None,
        Ar=None,
        meshsolution=-1,
        FEA_dict=None,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "Time" in list(init_dict.keys()):
                Time = init_dict["Time"]
            if "Angle" in list(init_dict.keys()):
                Angle = init_dict["Angle"]
            if "Nt_tot" in list(init_dict.keys()):
                Nt_tot = init_dict["Nt_tot"]
            if "Na_tot" in list(init_dict.keys()):
                Na_tot = init_dict["Na_tot"]
            if "logger_name" in list(init_dict.keys()):
                logger_name = init_dict["logger_name"]
            if "Yr" in list(init_dict.keys()):
                Yr = init_dict["Yr"]
            if "Vr" in list(init_dict.keys()):
                Vr = init_dict["Vr"]
            if "Ar" in list(init_dict.keys()):
                Ar = init_dict["Ar"]
            if "meshsolution" in list(init_dict.keys()):
                meshsolution = init_dict["meshsolution"]
            if "FEA_dict" in list(init_dict.keys()):
                FEA_dict = init_dict["FEA_dict"]
        # Set the properties (value check and convertion are done in setter)
        self.parent = None
        self.Time = Time
        self.Angle = Angle
        self.Nt_tot = Nt_tot
        self.Na_tot = Na_tot
        self.logger_name = logger_name
        self.Yr = Yr
        self.Vr = Vr
        self.Ar = Ar
        self.meshsolution = meshsolution
        self.FEA_dict = FEA_dict

        # The class is frozen, for now it's impossible to add new properties
        self._freeze()

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        OutStruct_str = ""
        if self.parent is None:
            OutStruct_str += "parent = None " + linesep
        else:
            OutStruct_str += "parent = " + str(type(self.parent)) + " object" + linesep
        OutStruct_str += "Time = " + str(self.Time) + linesep + linesep
        OutStruct_str += "Angle = " + str(self.Angle) + linesep + linesep
        OutStruct_str += "Nt_tot = " + str(self.Nt_tot) + linesep
        OutStruct_str += "Na_tot = " + str(self.Na_tot) + linesep
        OutStruct_str += 'logger_name = "' + str(self.logger_name) + '"' + linesep
        OutStruct_str += "Yr = " + str(self.Yr) + linesep + linesep
        OutStruct_str += "Vr = " + str(self.Vr) + linesep + linesep
        OutStruct_str += "Ar = " + str(self.Ar) + linesep + linesep
        if self.meshsolution is not None:
            tmp = (
                self.meshsolution.__str__()
                .replace(linesep, linesep + "\t")
                .rstrip("\t")
            )
            OutStruct_str += "meshsolution = " + tmp
        else:
            OutStruct_str += "meshsolution = None" + linesep + linesep
        OutStruct_str += "FEA_dict = " + str(self.FEA_dict) + linesep
        return OutStruct_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False
        if other.Time != self.Time:
            return False
        if other.Angle != self.Angle:
            return False
        if other.Nt_tot != self.Nt_tot:
            return False
        if other.Na_tot != self.Na_tot:
            return False
        if other.logger_name != self.logger_name:
            return False
        if other.Yr != self.Yr:
            return False
        if other.Vr != self.Vr:
            return False
        if other.Ar != self.Ar:
            return False
        if other.meshsolution != self.meshsolution:
            return False
        if other.FEA_dict != self.FEA_dict:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()
        if (other.Time is None and self.Time is not None) or (
            other.Time is not None and self.Time is None
        ):
            diff_list.append(name + ".Time None mismatch")
        elif self.Time is not None:
            diff_list.extend(self.Time.compare(other.Time, name=name + ".Time"))
        if (other.Angle is None and self.Angle is not None) or (
            other.Angle is not None and self.Angle is None
        ):
            diff_list.append(name + ".Angle None mismatch")
        elif self.Angle is not None:
            diff_list.extend(self.Angle.compare(other.Angle, name=name + ".Angle"))
        if other._Nt_tot != self._Nt_tot:
            diff_list.append(name + ".Nt_tot")
        if other._Na_tot != self._Na_tot:
            diff_list.append(name + ".Na_tot")
        if other._logger_name != self._logger_name:
            diff_list.append(name + ".logger_name")
        if (other.Yr is None and self.Yr is not None) or (
            other.Yr is not None and self.Yr is None
        ):
            diff_list.append(name + ".Yr None mismatch")
        elif self.Yr is not None:
            diff_list.extend(self.Yr.compare(other.Yr, name=name + ".Yr"))
        if (other.Vr is None and self.Vr is not None) or (
            other.Vr is not None and self.Vr is None
        ):
            diff_list.append(name + ".Vr None mismatch")
        elif self.Vr is not None:
            diff_list.extend(self.Vr.compare(other.Vr, name=name + ".Vr"))
        if (other.Ar is None and self.Ar is not None) or (
            other.Ar is not None and self.Ar is None
        ):
            diff_list.append(name + ".Ar None mismatch")
        elif self.Ar is not None:
            diff_list.extend(self.Ar.compare(other.Ar, name=name + ".Ar"))
        if (other.meshsolution is None and self.meshsolution is not None) or (
            other.meshsolution is not None and self.meshsolution is None
        ):
            diff_list.append(name + ".meshsolution None mismatch")
        elif self.meshsolution is not None:
            diff_list.extend(
                self.meshsolution.compare(
                    other.meshsolution, name=name + ".meshsolution"
                )
            )
        if other._FEA_dict != self._FEA_dict:
            diff_list.append(name + ".FEA_dict")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object
        S += getsizeof(self.Time)
        S += getsizeof(self.Angle)
        S += getsizeof(self.Nt_tot)
        S += getsizeof(self.Na_tot)
        S += getsizeof(self.logger_name)
        S += getsizeof(self.Yr)
        S += getsizeof(self.Vr)
        S += getsizeof(self.Ar)
        S += getsizeof(self.meshsolution)
        if self.FEA_dict is not None:
            for key, value in self.FEA_dict.items():
                S += getsizeof(value) + getsizeof(key)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        OutStruct_dict = dict()
        if self.Time is None:
            OutStruct_dict["Time"] = None
        else:
            OutStruct_dict["Time"] = self.Time.as_dict()
        if self.Angle is None:
            OutStruct_dict["Angle"] = None
        else:
            OutStruct_dict["Angle"] = self.Angle.as_dict()
        OutStruct_dict["Nt_tot"] = self.Nt_tot
        OutStruct_dict["Na_tot"] = self.Na_tot
        OutStruct_dict["logger_name"] = self.logger_name
        if self.Yr is None:
            OutStruct_dict["Yr"] = None
        else:
            OutStruct_dict["Yr"] = self.Yr.as_dict()
        if self.Vr is None:
            OutStruct_dict["Vr"] = None
        else:
            OutStruct_dict["Vr"] = self.Vr.as_dict()
        if self.Ar is None:
            OutStruct_dict["Ar"] = None
        else:
            OutStruct_dict["Ar"] = self.Ar.as_dict()
        if self.meshsolution is None:
            OutStruct_dict["meshsolution"] = None
        else:
            OutStruct_dict["meshsolution"] = self.meshsolution.as_dict(**kwargs)
        OutStruct_dict["FEA_dict"] = (
            self.FEA_dict.copy() if self.FEA_dict is not None else None
        )
        # The class name is added to the dict for deserialisation purpose
        OutStruct_dict["__class__"] = "OutStruct"
        return OutStruct_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.Time = None
        self.Angle = None
        self.Nt_tot = None
        self.Na_tot = None
        self.logger_name = None
        self.Yr = None
        self.Vr = None
        self.Ar = None
        if self.meshsolution is not None:
            self.meshsolution._set_None()
        self.FEA_dict = None

    def _get_Time(self):
        """getter of Time"""
        return self._Time

    def _set_Time(self, value):
        """setter of Time"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "SciDataTool.Classes", value.get("__class__"), "Time"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = Data()
        check_var("Time", value, "Data")
        self._Time = value

    Time = property(
        fget=_get_Time,
        fset=_set_Time,
        doc=u"""Structural time Data object

        :Type: SciDataTool.Classes.DataND.Data
        """,
    )

    def _get_Angle(self):
        """getter of Angle"""
        return self._Angle

    def _set_Angle(self, value):
        """setter of Angle"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "SciDataTool.Classes", value.get("__class__"), "Angle"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = Data()
        check_var("Angle", value, "Data")
        self._Angle = value

    Angle = property(
        fget=_get_Angle,
        fset=_set_Angle,
        doc=u"""Structural position Data object

        :Type: SciDataTool.Classes.DataND.Data
        """,
    )

    def _get_Nt_tot(self):
        """getter of Nt_tot"""
        return self._Nt_tot

    def _set_Nt_tot(self, value):
        """setter of Nt_tot"""
        check_var("Nt_tot", value, "int")
        self._Nt_tot = value

    Nt_tot = property(
        fget=_get_Nt_tot,
        fset=_set_Nt_tot,
        doc=u"""Length of the time vector

        :Type: int
        """,
    )

    def _get_Na_tot(self):
        """getter of Na_tot"""
        return self._Na_tot

    def _set_Na_tot(self, value):
        """setter of Na_tot"""
        check_var("Na_tot", value, "int")
        self._Na_tot = value

    Na_tot = property(
        fget=_get_Na_tot,
        fset=_set_Na_tot,
        doc=u"""Length of the angle vector

        :Type: int
        """,
    )

    def _get_logger_name(self):
        """getter of logger_name"""
        return self._logger_name

    def _set_logger_name(self, value):
        """setter of logger_name"""
        check_var("logger_name", value, "str")
        self._logger_name = value

    logger_name = property(
        fget=_get_logger_name,
        fset=_set_logger_name,
        doc=u"""Name of the logger to use

        :Type: str
        """,
    )

    def _get_Yr(self):
        """getter of Yr"""
        return self._Yr

    def _set_Yr(self, value):
        """setter of Yr"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "SciDataTool.Classes", value.get("__class__"), "Yr"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = DataND()
        check_var("Yr", value, "DataND")
        self._Yr = value

    Yr = property(
        fget=_get_Yr,
        fset=_set_Yr,
        doc=u"""Displacement output

        :Type: SciDataTool.Classes.DataND.DataND
        """,
    )

    def _get_Vr(self):
        """getter of Vr"""
        return self._Vr

    def _set_Vr(self, value):
        """setter of Vr"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "SciDataTool.Classes", value.get("__class__"), "Vr"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = DataND()
        check_var("Vr", value, "DataND")
        self._Vr = value

    Vr = property(
        fget=_get_Vr,
        fset=_set_Vr,
        doc=u"""Velocity output

        :Type: SciDataTool.Classes.DataND.DataND
        """,
    )

    def _get_Ar(self):
        """getter of Ar"""
        return self._Ar

    def _set_Ar(self, value):
        """setter of Ar"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "SciDataTool.Classes", value.get("__class__"), "Ar"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = DataND()
        check_var("Ar", value, "DataND")
        self._Ar = value

    Ar = property(
        fget=_get_Ar,
        fset=_set_Ar,
        doc=u"""Acceleration output

        :Type: SciDataTool.Classes.DataND.DataND
        """,
    )

    def _get_meshsolution(self):
        """getter of meshsolution"""
        return self._meshsolution

    def _set_meshsolution(self, value):
        """setter of meshsolution"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "meshsolution"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = MeshSolution()
        check_var("meshsolution", value, "MeshSolution")
        self._meshsolution = value

        if self._meshsolution is not None:
            self._meshsolution.parent = self

    meshsolution = property(
        fget=_get_meshsolution,
        fset=_set_meshsolution,
        doc=u"""FEA software mesh and solution

        :Type: MeshSolution
        """,
    )

    def _get_FEA_dict(self):
        """getter of FEA_dict"""
        return self._FEA_dict

    def _set_FEA_dict(self, value):
        """setter of FEA_dict"""
        if type(value) is int and value == -1:
            value = dict()
        check_var("FEA_dict", value, "dict")
        self._FEA_dict = value

    FEA_dict = property(
        fget=_get_FEA_dict,
        fset=_set_FEA_dict,
        doc=u"""Dictionnary containing the main FEA parameter

        :Type: dict
        """,
    )



# -*- coding: utf-8 -*-
"""File generated according to PCondType22/gen_list.json
WARNING! All changes made in this file will be lost!
"""
from pyleecan.GUI.Dialog.DMachineSetup.SBar.PCondType22.Ui_PCondType22 import (
    Ui_PCondType22,
)


class Gen_PCondType22(Ui_PCondType22):
    def setupUi(self, PCondType22):
        """Abstract class to update the widget according to the csv doc"""
        Ui_PCondType22.setupUi(self, PCondType22)
        # Setup of w_mat
        txt = self.tr(u"""Material of the conductor""")
        self.w_mat.setWhatsThis(txt)
        self.w_mat.setToolTip(txt)



# -*- coding: utf-8 -*-
from numpy import pi

from ....Classes.Winding import Winding
from ....Methods import NotImplementedYetError
from ....Classes.LamSlot import LamSlot


def build_geometry(self, sym=1, alpha=0, delta=0, is_simplified=False):
    """Build the geometry of the Lamination with winding in slots

    Parameters
    ----------
    self : LamSlotWind
        A LamSlotWind object
    sym : int
        Symmetry factor (1= full machine, 2= half of the machine...)
    alpha : float
        Angle for rotation [rad]
    delta : complex
        Complex value for translation
    is_simplified: bool
        True to avoid line superposition

    Returns
    -------
    surf_list : list
        list of surfaces needed to draw the lamination

    """
    # getting the Lamination surface
    surf_lam = LamSlot.build_geometry(self, sym=sym, alpha=alpha, delta=delta)
    surf_list = list()
    if self.slot is not None and self.slot.Zs != 0:
        # getting number of slot
        Zs = self.slot.Zs
        # getting angle between Slot
        angle = 2 * pi / Zs
        # getting Nrad and Ntan
        if (type(self.winding) is Winding) or (self.winding is None):
            Nrad, Ntan = 1, 1
            surf_Wind = list()
        else:
            try:
                Nrad, Ntan = self.winding.get_dim_wind()
            except NotImplementedYetError:
                Nrad, Ntan = 1, 1

            surf_Wind = self.slot.build_geometry_active(
                Nrad=Nrad,
                Ntan=Ntan,
                is_simplified=is_simplified,
                alpha=alpha,
                delta=delta,
            )

        if self.is_stator:
            st = "Stator"
        else:
            st = "Rotor"
        assert (self.slot.Zs % sym) == 0, (
            "ERROR, Wrong symmetry for "
            + st
            + " "
            + str(self.slot.Zs)
            + " slots and sym="
            + str(sym)
        )
        for ii in range(Zs // sym):  # for each slot
            # for each part of the winding surface in the slot
            for surf in surf_Wind:
                new_surf = type(surf)(init_dict=surf.as_dict())
                # changing the slot reference number
                new_surf.label = surf.label[:-1] + str(ii)
                new_surf.rotate(ii * angle)
                surf_list.append(new_surf)

        # Shift to have a tooth center on Ox
        for surf in surf_list:
            surf.rotate(pi / Zs)

    surf_list = surf_lam + surf_list

    return surf_list



# -*- coding: utf-8 -*-


def reverse_wind_mat(wind_mat):
    """reverse a Winding Matrix along the slot

    Parameters
    ----------
    wind_mat : numpy.ndarray
        A Winding Matrix (Nlay_r, Nlay_theta, Zs, qs)
        (created by comp_connection_mat)

    Returns
    -------
    wind_mat: numpy.ndarray
        The reverse matrix along the Slot

    """
    assert len(wind_mat.shape) == 4, "wind_mat has a wrong shape (dim 4)"

    wind_mat = wind_mat[:, :, ::-1, :]
    if wind_mat.shape[1] == 2:
        wind_mat = wind_mat[:, ::-1, :, :]

    return wind_mat



# -*- coding: utf-8 -*-
from os.path import join, dirname, split
from PySide2.QtWidgets import QDialog, QMessageBox, QFileDialog
from PySide2.QtCore import Qt

from .....GUI.Dialog.DMatLib.DMatSetup.Gen_DMatSetup import Gen_DMatSetup
from .....GUI.Tools.MPLCanvas import MPLCanvas

from .....Classes.Material import Material
from .....Classes.MatMagnetics import MatMagnetics
from .....Classes.ImportMatrixXls import ImportMatrixXls
from .....Classes.ImportMatrixVal import ImportMatrixVal

from .....Functions.path_tools import abs_file_path, rel_file_path

from numpy import array


class DMatSetup(Gen_DMatSetup, QDialog):
    def __init__(self, material, is_matlib=True):
        """
        Dialog for editing material data.

        Parameters
        ----------
        material : Material
            material to edit
        is_matlib : bool
            material already in matlib
        """
        # Build the interface according to the .ui file
        QDialog.__init__(self)
        self.setupUi(self)

        self.is_save_needed = False

        # Copy to set the modification only if validated
        self.mat = Material(init_dict=material.as_dict())

        self.le_name.setText(self.mat.name)
        if self.mat.is_isotropic:
            self.is_isotropic.setCheckState(Qt.Checked)
            self.nav_meca.setCurrentIndex(1)
            self.nav_ther.setCurrentIndex(1)
        else:
            self.is_isotropic.setCheckState(Qt.Unchecked)
            self.nav_meca.setCurrentIndex(0)
            self.nav_ther.setCurrentIndex(0)

        # Edit button text if the Material selected is in the ref matlib
        if is_matlib:
            self.b_add_matlib.setText("Add to machine")

        # Three button to close
        self.b_cancel.clicked.connect(lambda: self.done(0))
        self.b_save.clicked.connect(lambda: self.done(1))
        self.b_add_matlib.clicked.connect(lambda: self.done(2))

        # === check material attribute and set values ===
        # Elec
        if self.mat.elec is None:
            self.set_default("elec", "electrical")
        self.lf_rho_elec.setValue(self.mat.elec.rho)

        # Economical
        if self.mat.eco is None:
            self.set_default("eco", "economical")
        self.lf_cost_unit.setValue(self.mat.eco.cost_unit)

        # Thermics
        if self.mat.HT is None:
            self.set_default("HT", "thermaical")
        self.lf_Cp.setValue(self.mat.HT.Cp)
        self.lf_alpha.setValue(self.mat.HT.alpha)
        self.lf_L.setValue(self.mat.HT.lambda_x)
        self.lf_Lx.setValue(self.mat.HT.lambda_x)
        self.lf_Ly.setValue(self.mat.HT.lambda_y)
        self.lf_Lz.setValue(self.mat.HT.lambda_z)
        # Structural
        if self.mat.struct is None:
            self.set_default("struct", "structural")
        self.lf_rho_meca.setValue(self.mat.struct.rho)
        self.lf_E.setValue(self.mat.struct.Ex)
        self.lf_Ex.setValue(self.mat.struct.Ex)
        self.lf_Ey.setValue(self.mat.struct.Ey)
        self.lf_Ez.setValue(self.mat.struct.Ez)
        self.lf_G.setValue(self.mat.struct.Gxy)
        self.lf_Gxy.setValue(self.mat.struct.Gxy)
        self.lf_Gxz.setValue(self.mat.struct.Gxz)
        self.lf_Gyz.setValue(self.mat.struct.Gyz)
        self.lf_nu.setValue(self.mat.struct.nu_xy)
        self.lf_nu_xy.setValue(self.mat.struct.nu_xy)
        self.lf_nu_xz.setValue(self.mat.struct.nu_xz)
        self.lf_nu_yz.setValue(self.mat.struct.nu_yz)

        # Magnetical
        if self.mat.mag is None:
            self.set_default("mag", "magnetical")
        self.lf_mur_lin.setValue(self.mat.mag.mur_lin)
        self.lf_Brm20.setValue(self.mat.mag.Brm20)
        self.lf_alpha_Br.setValue(self.mat.mag.alpha_Br)
        self.lf_Wlam.setValue(self.mat.mag.Wlam)
        # Setup import B(H) widget
        self.w_BH_import.verbose_name = "B(H) curve"
        self.w_BH_import.obj = self.mat.mag
        self.w_BH_import.param_name = "BH_curve"
        self.w_BH_import.expected_shape = (None, 2)
        self.w_BH_import.update()

        # Hide useless widget
        self.in_epsr.hide()
        self.lf_epsr.hide()

        # === setup signals ===
        # Misc.
        self.le_name.editingFinished.connect(self.set_name)
        self.is_isotropic.toggled.connect(self.set_is_isotropic)
        self.lf_rho_elec.editingFinished.connect(self.set_rho_elec)
        # Magnetics
        self.lf_mur_lin.editingFinished.connect(self.set_mur_lin)
        self.lf_Brm20.editingFinished.connect(self.set_Brm20)
        self.lf_alpha_Br.editingFinished.connect(self.set_alpha_Br)
        self.lf_Wlam.editingFinished.connect(self.set_Wlam)
        # Economical
        self.lf_cost_unit.editingFinished.connect(self.set_cost_unit)
        # Thermics
        self.lf_Cp.editingFinished.connect(self.set_Cp)
        self.lf_alpha.editingFinished.connect(self.set_alpha)
        self.lf_L.editingFinished.connect(self.set_lambda)
        self.lf_Lx.editingFinished.connect(self.set_lambda_x)
        self.lf_Ly.editingFinished.connect(self.set_lambda_y)
        self.lf_Lz.editingFinished.connect(self.set_lambda_z)
        # Mechanics
        self.lf_rho_meca.editingFinished.connect(self.set_rho_meca)
        self.lf_E.editingFinished.connect(self.set_E)
        self.lf_Ex.editingFinished.connect(self.set_Ex)
        self.lf_Ey.editingFinished.connect(self.set_Ey)
        self.lf_Ez.editingFinished.connect(self.set_Ez)
        self.lf_G.editingFinished.connect(self.set_G)
        self.lf_Gxy.editingFinished.connect(self.set_Gxy)
        self.lf_Gxz.editingFinished.connect(self.set_Gxz)
        self.lf_Gyz.editingFinished.connect(self.set_Gyz)
        self.lf_nu.editingFinished.connect(self.set_nu)
        self.lf_nu_xy.editingFinished.connect(self.set_nu_xy)
        self.lf_nu_xz.editingFinished.connect(self.set_nu_xz)
        self.lf_nu_yz.editingFinished.connect(self.set_nu_yz)

    def closeEvent(self, event):
        """Display a message before leaving

        Parameters
        ----------
        self : DMatSetup
            A DMatSetup object
        event :
            The closing event
        """

        if self.is_save_needed:
            quit_msg = self.tr(
                "Unsaved changes will be lost.\nDo you want to save the material?"
            )
            reply = QMessageBox.question(
                self,
                self.tr("Please save before closing"),
                quit_msg,
                QMessageBox.Yes,
                QMessageBox.No,
            )
            self.qmessagebox_question = reply
            if reply == QMessageBox.Yes:
                self.done(1)
            else:
                self.done(0)
        else:
            self.done(0)

    def set_default(self, attr, attr_name):
        msg = QMessageBox()
        msg.setIcon(QMessageBox.Critical)
        msg.setText(
            "Material "
            + attr_name
            + " property is None.\nDefault values set. Please check values."
        )
        msg.setWindowTitle("Warning")
        msg.exec_()
        setattr(self.mat, attr, type(getattr(Material(), attr))())

    def set_name(self):
        """Signal to update the value of name according to the line edit

        Parameters
        ----------
        self : DMatSetup
            A DMatSetup object

        Returns
        -------
        None
        """

        file_name = str(self.le_name.text())
        # Update name and path
        self.mat.name = file_name
        self.le_name.setText(self.mat.name)
        self.mat.path = rel_file_path(
            join(dirname(self.mat.path), file_name + ".json"), "MATLIB_DIR"
        )
        self.is_save_needed = True

    def set_is_isotropic(self, is_checked):
        """Signal to update the value of is_isotropic according to the checkbox

        Parameters
        ----------
        self :
            A DMatSetup object
        is_checked :
            State of the checkbox

        Returns
        -------
        None
        """
        self.mat.is_isotropic = is_checked
        self.nav_meca.setCurrentIndex(int(is_checked))
        self.nav_ther.setCurrentIndex(int(is_checked))

    def set_rho_elec(self):
        """Signal to update the value of rho_elec according to the line edit

        Parameters
        ----------
        self :
            A DMatSetup object

        Returns
        -------
        None
        """
        self.mat.elec.rho = self.lf_rho_elec.value()
        self.is_save_needed = True

    def set_mur_lin(self):
        """Signal to update the value of mur_lin according to the line edit

        Parameters
        ----------
        self :
            A DMatSetup object

        Returns
        -------
        None
        """
        self.mat.mag.mur_lin = self.lf_mur_lin.value()
        self.is_save_needed = True

    def set_Brm20(self):
        """Signal to update the value of Brm20 according to the line edit

        Parameters
        ----------
        self :
            A DMatSetup object

        Returns
        -------
        None
        """
        self.mat.mag.Brm20 = self.lf_Brm20.value()
        self.is_save_needed = True

    def set_alpha_Br(self):
        """Signal to update the value of alpha_Br according to the line edit

        Parameters
        ----------
        self :
            A DMatSetup object

        Returns
        -------
        None
        """
        self.mat.mag.alpha_Br = self.lf_alpha_Br.value()
        self.is_save_needed = True

    def set_Wlam(self):
        """Signal to update the value of Wlam according to the line edit

        Parameters
        ----------
        self :
            A DMatSetup object

        Returns
        -------
        None
        """
        self.mat.mag.Wlam = self.lf_Wlam.value()
        self.is_save_needed = True

    def set_cost_unit(self):
        """Signal to update the value of cost_unit according to the line edit

        Parameters
        ----------
        self :
            A DMatSetup object

        Returns
        -------
        None
        """
        self.mat.eco.cost_unit = self.lf_cost_unit.value()
        self.is_save_needed = True

    def set_Cp(self):
        """Signal to update the value of Cp according to the line edit

        Parameters
        ----------
        self :
            A DMatSetup object

        Returns
        -------
        None
        """
        self.mat.HT.Cp = self.lf_Cp.value()
        self.is_save_needed = True

    def set_alpha(self):
        """Signal to update the value of alpha according to the line edit

        Parameters
        ----------
        self :
            A DMatSetup object

        Returns
        -------
        None
        """
        self.mat.HT.alpha = self.lf_alpha.value()
        self.is_save_needed = True

    def set_lambda(self):
        """Signal to update the value of lambda according to the line edit

        Parameters
        ----------
        self :
            A DMatSetup object

        Returns
        -------
        None
        """
        self.mat.HT.lambda_x = self.lf_L.value()
        self.mat.HT.lambda_y = self.lf_L.value()
        self.mat.HT.lambda_z = self.lf_L.value()
        self.is_save_needed = True

    def set_lambda_x(self):
        """Signal to update the value of lambda_x according to the line edit

        Parameters
        ----------
        self :
            A DMatSetup object

        Returns
        -------
        None
        """
        self.mat.HT.lambda_x = self.lf_Lx.value()
        self.is_save_needed = True

    def set_lambda_y(self):
        """Signal to update the value of lambda_y according to the line edit

        Parameters
        ----------
        self :
            A DMatSetup object

        Returns
        -------
        None
        """
        self.mat.HT.lambda_y = self.lf_Ly.value()
        self.is_save_needed = True

    def set_lambda_z(self):
        """Signal to update the value of lambda_z according to the line edit

        Parameters
        ----------
        self :
            A DMatSetup object

        Returns
        -------
        None
        """
        self.mat.HT.lambda_z = self.lf_Lz.value()
        self.is_save_needed = True

    def set_rho_meca(self):
        """Signal to update the value of rho_meca according to the line edit

        Parameters
        ----------
        self :
            A DMatSetup object

        Returns
        -------
        None
        """
        self.mat.struct.rho = self.lf_rho_meca.value()
        self.is_save_needed = True

    def set_E(self):
        """Signal to update the value of Ex according to the line edit

        Parameters
        ----------
        self :
            A DMatSetup object

        Returns
        -------
        None
        """
        self.mat.struct.Ex = self.lf_E.value()
        self.mat.struct.Ey = self.lf_E.value()
        self.mat.struct.Ez = self.lf_E.value()
        self.is_save_needed = True

    def set_Ex(self):
        """Signal to update the value of Ex according to the line edit

        Parameters
        ----------
        self :
            A DMatSetup object

        Returns
        -------
        None
        """
        self.mat.struct.Ex = self.lf_Ex.value()
        self.is_save_needed = True

    def set_Ey(self):
        """Signal to update the value of Ey according to the line edit

        Parameters
        ----------
        self :
            A DMatSetup object

        Returns
        -------
        None
        """
        self.mat.struct.Ey = self.lf_Ey.value()
        self.is_save_needed = True

    def set_Ez(self):
        """Signal to update the value of Ez according to the line edit

        Parameters
        ----------
        self :
            A DMatSetup object

        Returns
        -------
        None
        """
        self.mat.struct.Ez = self.lf_Ez.value()
        self.is_save_needed = True

    def set_G(self):
        """Signal to update the value of G according to the line edit

        Parameters
        ----------
        self :
            A DMatSetup object

        Returns
        -------
        None
        """
        self.mat.struct.Gxy = self.lf_G.value()
        self.mat.struct.Gxz = self.lf_G.value()
        self.mat.struct.Gyz = self.lf_G.value()
        self.is_save_needed = True

    def set_Gxy(self):
        """Signal to update the value of Gxy according to the line edit

        Parameters
        ----------
        self :
            A DMatSetup object

        Returns
        -------
        None
        """
        self.mat.struct.Gxy = self.lf_Gxy.value()
        self.is_save_needed = True

    def set_Gxz(self):
        """Signal to update the value of Gxz according to the line edit

        Parameters
        ----------
        self :
            A DMatSetup object

        Returns
        -------
        None
        """
        self.mat.struct.Gxz = self.lf_Gxz.value()
        self.is_save_needed = True

    def set_Gyz(self):
        """Signal to update the value of Gyz according to the line edit

        Parameters
        ----------
        self :
            A DMatSetup object

        Returns
        -------
        None
        """
        self.mat.struct.Gyz = self.lf_Gyz.value()
        self.is_save_needed = True

    def set_nu(self):
        """Signal to update the value of nu_xy according to the line edit

        Parameters
        ----------
        self :
            A DMatSetup object

        Returns
        -------
        None
        """
        self.mat.struct.nu_xy = self.lf_nu.value()
        self.mat.struct.nu_xz = self.lf_nu.value()
        self.mat.struct.nu_yz = self.lf_nu.value()
        self.is_save_needed = True

    def set_nu_xy(self):
        """Signal to update the value of nu_xy according to the line edit

        Parameters
        ----------
        self :
            A DMatSetup object

        Returns
        -------
        None
        """
        self.mat.struct.nu_xy = self.lf_nu_xy.value()
        self.is_save_needed = True

    def set_nu_xz(self):
        """Signal to update the value of nu_xz according to the line edit

        Parameters
        ----------
        self :
            A DMatSetup object

        Returns
        -------
        None
        """
        self.mat.struct.nu_xz = self.lf_nu_xz.value()
        self.is_save_needed = True

    def set_nu_yz(self):
        """Signal to update the value of nu_yz according to the line edit

        Parameters
        ----------
        self :
            A DMatSetup object

        Returns
        -------
        None
        """
        self.mat.struct.nu_yz = self.lf_nu_yz.value()
        self.is_save_needed = True



import numpy as np
import matplotlib.pyplot as plt
from matplotlib.colors import LinearSegmentedColormap
from ....Methods.Output.XOutput import _get_symbol_data_


def plot_generation(self, x_symbol, y_symbol, ax=None):
    """Plot every fitness values according to the two fitness

    Parameters
    ----------
    self : XOutput
    obj1 : str
        symbol of the ParamExplorer, the OptiObjective or the DataKeeper
    obj2 : str
        symbol of the ParamExplorer, the OptiObjective or the DataKeeper
    """

    # TODO define the colormap according to Pyleecan graphical chart
    # Colormap definition
    cm = LinearSegmentedColormap.from_list(
        "colormap",
        [(35 / 255, 89 / 255, 133 / 255), (250 / 255, 202 / 255, 56 / 255)],
        N=max(self["ngen"].result) + 1,
    )

    # Get fitness and ngen
    is_valid = np.array(self["is_valid"].result)
    ngen = np.array(self["ngen"].result)

    # Keep only valid values
    indx = np.where(is_valid)[0]

    ngen = ngen[indx]

    # get data and labels
    x_values, x_label = _get_symbol_data_(self, x_symbol, indx)
    y_values, y_label = _get_symbol_data_(self, y_symbol, indx)

    if ax is None:
        fig, ax = plt.subplots()

        # Plot fitness values
        scatter = ax.scatter(x_values, y_values, s=8, c=ngen, cmap=cm)

        # Add legend
        legend1 = ax.legend(
            *scatter.legend_elements(), loc="upper right", title="Generation"
        )
        ax.add_artist(legend1)

        # Extend xlim to give some space to the legend
        left, right = ax.get_xlim()
        ax.set_xlim(left, right + 0.2 * abs(right - left))

        ax.set_xlabel(x_label)
        ax.set_ylabel(y_label)
        ax.set_title("Fitness values for each individual")
        fig.show()

    else:
        # Plot fitness values
        scatter = ax.scatter(x_values, y_values, s=8, c=ngen, cmap=cm)

        # Add legend
        legend1 = ax.legend(
            *scatter.legend_elements(), loc="upper right", title="Generation"
        )
        ax.add_artist(legend1)

        # Extend xlim to give some space to the legend
        left, right = ax.get_xlim()
        ax.set_xlim(left, right + 0.2 * abs(right - left))

        ax.set_xlabel(x_label)
        ax.set_ylabel(y_label)
        ax.set_title("Fitness values for each individual")

        return ax



# -*- coding: utf-8 -*-

from numpy import arcsin, cos


def comp_height_active(self):
    """Compute the height of the winding area

    Parameters
    ----------
    self : SlotW26
        A SlotW26 object

    Returns
    -------
    Hwind: float
        Height of the winding area [m]

    """

    # Height of the arc (P2,C1,P7)
    alpha2 = arcsin(self.W0 / (2.0 * self.R1))
    Harc2 = float(self.R1 * (1 - cos(alpha2)))

    return self.H1 + self.R2 + self.R1 - Harc2



# -*- coding: utf-8 -*-
"""File generated according to PWSlot12/gen_list.json
WARNING! All changes made in this file will be lost!
"""
from pyleecan.GUI.Dialog.DMachineSetup.SWSlot.PWSlot12.Ui_PWSlot12 import Ui_PWSlot12


class Gen_PWSlot12(Ui_PWSlot12):
    def setupUi(self, PWSlot12):
        """Abstract class to update the widget according to the csv doc"""
        Ui_PWSlot12.setupUi(self, PWSlot12)
        # Setup of in_R1
        txt = self.tr(u"""Wedges radius""")
        self.in_R1.setWhatsThis(txt)
        self.in_R1.setToolTip(txt)

        # Setup of lf_R1
        self.lf_R1.validator().setBottom(0)
        txt = self.tr(u"""Wedges radius""")
        self.lf_R1.setWhatsThis(txt)
        self.lf_R1.setToolTip(txt)

        # Setup of in_R2
        txt = self.tr(u"""Slot bottom radius""")
        self.in_R2.setWhatsThis(txt)
        self.in_R2.setToolTip(txt)

        # Setup of lf_R2
        self.lf_R2.validator().setBottom(0)
        txt = self.tr(u"""Slot bottom radius""")
        self.lf_R2.setWhatsThis(txt)
        self.lf_R2.setToolTip(txt)

        # Setup of in_H0
        txt = self.tr(u"""Slot isthmus height.""")
        self.in_H0.setWhatsThis(txt)
        self.in_H0.setToolTip(txt)

        # Setup of lf_H0
        self.lf_H0.validator().setBottom(0)
        txt = self.tr(u"""Slot isthmus height.""")
        self.lf_H0.setWhatsThis(txt)
        self.lf_H0.setToolTip(txt)

        # Setup of in_H1
        txt = self.tr(u"""Slot middle height""")
        self.in_H1.setWhatsThis(txt)
        self.in_H1.setToolTip(txt)

        # Setup of lf_H1
        self.lf_H1.validator().setBottom(0)
        txt = self.tr(u"""Slot middle height""")
        self.lf_H1.setWhatsThis(txt)
        self.lf_H1.setToolTip(txt)



# -*- coding: utf-8 -*-

from numpy import pi
from ....Classes.Lamination import Lamination


def comp_surfaces(self):
    """Compute the Lamination surfaces

    Parameters
    ----------
    self : LamHole
        A LamHole object

    Returns
    -------
    S_dict: dict
        Lamination surface dictionnary (Slam, Svent, Smag, Shole) [m**2]

    """

    S_dict = Lamination.comp_surfaces(self)

    # hole surface
    Shole = 0
    Smag = 0
    for hole in self.hole:
        Shole += hole.Zh * hole.comp_surface()
        if hole.has_magnet():
            Smag += hole.Zh * hole.comp_surface_magnets()

    Ryoke = self.get_Ryoke()
    Hyoke = self.comp_height_yoke()
    if self.is_internal:
        S_dict["Syoke"] = pi * ((Ryoke + Hyoke) ** 2 - Ryoke ** 2) - S_dict["Svent"]
    else:
        S_dict["Syoke"] = pi * (Ryoke ** 2 - (Ryoke - Hyoke) ** 2) - S_dict["Svent"]

    S_dict["Smag"] = Smag
    S_dict["Shole"] = Shole
    S_dict["Slam"] -= Shole  # the magnet surface in included in the hole one
    S_dict["Steeth"] = S_dict["Slam"] - S_dict["Syoke"]

    return S_dict
